local config = require 'system_config'
local network_db = require 'network_db'
local cjson = require 'cjson.safe'

local net_db = network_db:new()

local configuration_wanted = 'non-gps time'
if config.gps_timing_regulation and
    config.gps_timing_regulation == 'sync_to_gps_time' then
    configuration_wanted = 'gps time'
end

local function get_unit_sync_configuration(unit)
    local rs = net_db:query(
        "SELECT owner FROM unified WHERE type = 'SRV' AND owner = 'gps." ..
        unit .. "'")
    if not rs or type(rs) ~= 'table' or #rs < 1 then return end

    rs = net_db:query(
        "SELECT rdata FROM unified WHERE type = 'TXT' AND rdata LIKE " ..
        "'gps_time_synced:%' AND owner = 'gps." .. unit .. "'")

    if not rs or type(rs) ~= 'table' or #rs < 1 then
        return true, 'non-gps time', true
    end

    return true, 'gps time', rs[1].rdata:find('true') and true or false
end

local function get_unit_configurations(list)
    local info = {}
    for _, unit in ipairs(list) do
        local mode = 'non-gps time'
        local synced = true
        local unit_exists = false

        unit_exists, mode, synced = get_unit_sync_configuration(unit)

        if unit_exists then
            info[unit] = {
                mode = mode,
                synced = synced
            }
        end
    end
    return info
end

local function check_sync_configuration(unit_list)
    if not unit_list or type(unit_list) ~= 'table' or #unit_list < 1 then
        return
    end
    local misconfigured, not_locked

    local unit_sync_configurations = get_unit_configurations(unit_list)

    -- Generate a list of devices that do not have the same synchronization
    -- configuration as this unit
    for _, k in ipairs(unit_list) do
        if not unit_sync_configurations[k] or
            unit_sync_configurations[k].mode ~= configuration_wanted then
            misconfigured = misconfigured or {}
            misconfigured[#misconfigured + 1] = k
        end
    end

    -- If desired sync mode is NOT GPS Time units are always considered locked.
    -- Otherwise units must have GPS time sync
    if configuration_wanted == 'gps time' then
        -- Generate a list of units that are not GPS time synchronized
        for _, k in ipairs(unit_list) do
            if not unit_sync_configurations[k] or
                not unit_sync_configurations[k].synced then
                not_locked = not_locked or {}
                not_locked[#not_locked + 1] = k
            end
        end
    end

    return misconfigured, not_locked
end

local function check_sync(preprocessed_setup)
    if type(preprocessed_setup) == 'string' then
        local fh = io.open(preprocessed_setup, 'r')
        if not fh then
            error("check_sync: Can not open file " .. preprocessed_setup)
            return
        end
        local sxrpp = fh:read('*a')
        fh:close()
        if not sxrpp then
            error("check_sync: Can not read file " .. preprocessed_setup)
            return
        end
        preprocessed_setup = cjson.decode(sxrpp)
        if not preprocessed_setup or not preprocessed_setup.sxr then
            error("check_sync: Unable to decode SXRPP file")
            return
        end
    elseif type(preprocessed_setup) ~= 'table' then
        error("check_sync: Unexpected argument type - " ..
            type(preprocessed_setup))
    end

    local net_nodes = {}
    local unit_list = {}
    for _, stanza in ipairs(preprocessed_setup.sxr) do
        if stanza.network_node then
            if not net_nodes[stanza.network_node] then
                unit_list[#unit_list + 1] = stanza.network_node
                net_nodes[stanza.network_node] = true
            end
        end
    end

    -- If there is only one unit and it is this unit we don't need to wait for
    -- time sync to lock.
    if #unit_list < 1 then
        return
    elseif #unit_list == 1 and
        net_db:get_inside_network_hostname() == unit_list[1] then
        return
    end

    return check_sync_configuration(unit_list)
end

return check_sync
