-- gps package
--
-- Provides an API to a GPS channel
--
-- API Functions:
--  gps:new(devname, firmware_path, master_freq)
--  gps:poll()
--  gps:ts_offset(ts)
--  gps:setup_sync_ts()
--  gps:get_sync_ts()
--  gps:get_current_ts() [DEBUG]
--  gps:txdr_power_on_5v()
--  gps:txdr_power_on_12v()
--  gps:txdr_power_off()
--  gps:uart_setup(uart_setting, baud_rate) [DEBUG]
--  gps:uart_rx_start(start_time, heartbeat_interval,
--                    samples_per_bd, num_bds,
--                    uart_setting, baud_rate)
--  gps:uart_rx_stop()
--  gps:uart_tx_start(num_buffers)
--  gps:uart_tx_char(c) [DEBUG]
--  gps:uart_tx_data(databytes)
--  gps:uart_tx_stop()
--  gps:event_setup(args)
--  gps:event_start(start_time, heartbeat_interval,
--                  samples_per_bd, num_bds)
--  gps:event_stop()
--  gps:set_rts_state(new_state)
--  gps:get_cts_state()
--  gps:get_cable_detect_state()
--
--
-- Notes on event_setup(...) parameters:
--   Set the following parameter names to true or non-zero to enable.
--       cable_unplugged
--       cable_plugged
--       cts_deassert
--       cts_assert
--       pps_falling
--       pps_rising
--       end_of_break
--       start_of_break
--       parity_error
--       frame_error
--
--   For example: event_setup({pps_rising = true})
--   All parameters not listed are disabled by default.
--


local cc = require 'connctrl'
local ffi = require 'ffi'
local rb = require 'ring_buffers'
local ch = require 'channel'
local config = require 'system_config'

local gps = setmetatable({}, { __index = ch })
local f = assert(io.open("/usr/include/fpga-firmware/gps_structures.h", "r"))
local h = f:read("*all")
f:close()
ffi.cdef(h)


function gps:new(devname, firmware_path, master_freq)
    local gps_ch = self:new_channel(devname, firmware_path, "gps_cmd_s")
    gps_ch.master_freq = master_freq
    return gps_ch
end

function gps:ts_offset(ts)
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_TS_OFFSET
    gpscmd.prm.ts_offset.ts_adjust = ts
    return self:send_command_buffer(gpscmd)
end

function gps:setup_sync_ts()
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_SETUP_SYNC_TS
    return self:send_command_buffer(gpscmd)
end

function gps:get_sync_ts()
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_GET_SYNC_TS
    return self:send_command_buffer(gpscmd)
end

function gps:get_current_ts()
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_GET_TS
    return self:send_command_buffer(gpscmd)
end

function gps:txdr_power_on_5v()
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_TXDR_POWER_ON_5V
    return self:send_command_buffer(gpscmd)
end

function gps:txdr_power_on_12v()
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_TXDR_POWER_ON_12V
    return self:send_command_buffer(gpscmd)
end

function gps:txdr_power_off()
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_TXDR_POWER_OFF
    return self:send_command_buffer(gpscmd)
end

function gps:uart_setup(uart_setting, baud_rate)
    local smpl_freq = baud_rate * 5     -- 5X oversampled clock required
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_UART_SETUP
    self:calculate_scg_parameters(gpscmd.prm.uart_setup.baud_rate,
                                  baud_rate, 1, self.master_freq)
    ffi.copy(gpscmd.prm.uart_setup.uart_setting, uart_setting, 4)
    return self:send_command_buffer(gpscmd)
end

function gps:uart_rx_start(start_time, heartbeat_interval,
                           samples_per_bd, num_bds,
                           uart_setting, baud_rate)
    local pa = self:add_ring_buffer('RX', "rx_buffer_t["..samples_per_bd.."]",
                                    "rx_buffer_t", num_bds)
    local smpl_freq = baud_rate * 5     -- 5X oversampled clock required
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_UART_RX_START
    self:calculate_scg_parameters(gpscmd.prm.rx_start.baud_rate,
                                  smpl_freq, 1, self.master_freq)
    ffi.copy(gpscmd.prm.rx_start.uart_setting, uart_setting, 4)
    gpscmd.prm.rx_start.phys_address = pa
    gpscmd.prm.rx_start.start_time = start_time
    self:calculate_scg_parameters(gpscmd.prm.rx_start.heartbeat_interval,
                                  heartbeat_interval, 1, self.master_freq)
    return self:send_command_buffer(gpscmd)
end

function gps:uart_rx_stop()
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_UART_RX_STOP
    return self:send_command_buffer(gpscmd)
end

function gps:uart_tx_start(num_buffers)
    local pa = self:add_ring_buffer('TX', "tx_buffer_t",
                                    "tx_buffer_t", num_buffers)
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_UART_TX_START
    gpscmd.prm.tx_start.phys_address = pa
    return self:send_command_buffer(gpscmd)
end

function gps:uart_tx_char(c)
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_UART_TX_CHAR
    gpscmd.prm.tx_char.txdata = string.byte(c, 1)
    return self:send_command_buffer(gpscmd)
end

function gps:uart_tx_data(databytes)
    local s = databytes
    while #s > 0 do
        local buf = ffi.new("tx_buffer_t")
        local bytes_to_copy = ffi.sizeof("tx_buffer_t")
        if #s < bytes_to_copy then bytes_to_copy = #s end
        ffi.copy(buf, s, bytes_to_copy)
        if not self:send_buffer(buf, 'TX', bytes_to_copy) then
            return s
        end
        s = s:sub(bytes_to_copy + 1)
    end
end

function gps:uart_tx_stop()
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_UART_TX_STOP
    return self:send_command_buffer(gpscmd)
end

function gps:event_setup(arg)
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_EVENT_SETUP
    if arg.cable_unplugged then gpscmd.prm.events.cable_unplugged = 1 end
    if arg.cable_plugged then gpscmd.prm.events.cable_plugged = 1 end
    if arg.cts_deassert then gpscmd.prm.events.cts_inactive = 1 end
    if arg.cts_assert then gpscmd.prm.events.cts_active = 1 end
    if arg.pps_falling then gpscmd.prm.events.pps_falling = 1 end
    if arg.pps_rising then gpscmd.prm.events.pps_rising = 1 end
    if arg.end_of_break then gpscmd.prm.events.end_of_break = 1 end
    if arg.start_of_break then gpscmd.prm.events.start_of_break = 1 end
    if arg.parity_error then gpscmd.prm.events.parity_error = 1 end
    if arg.frame_error then gpscmd.prm.events.frame_error = 1 end
    return self:send_command_buffer(gpscmd)
end

function gps:event_start(start_time, heartbeat_interval,
                         samples_per_bd, num_bds)
    local pa = self:add_ring_buffer('EV',
                                    "gps_ev_buffer_t["..samples_per_bd.."]",
                                    "gps_ev_buffer_t", num_bds)
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_EVENT_START
    gpscmd.prm.ev_start.phys_address = pa
    gpscmd.prm.ev_start.start_time = start_time
    self:calculate_scg_parameters(gpscmd.prm.ev_start.heartbeat_interval,
                                  heartbeat_interval, 1, self.master_freq)
    return self:send_command_buffer(gpscmd)
end

function gps:event_stop()
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_EVENT_STOP
    return self:send_command_buffer(gpscmd)
end

function gps:set_rts_state(new_state)
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_FLOW_CONTROL
    gpscmd.prm.flow_control.update_rts = 1;
    gpscmd.prm.flow_control.rts_state = new_state;
    return self:send_command_buffer(gpscmd)
end

function gps:get_cts_state()
    local gpscmd = ffi.new("gps_cmd_s")
    gpscmd.cmd = ffi.C.GPS_FLOW_CONTROL
    gpscmd.prm.flow_control.update_rts = 0;
    return self:send_command_buffer(gpscmd)
end

if config.device_type == 'cx23r' then
    function gps:get_cable_detect_state()
        local gpscmd = ffi.new("gps_cmd_s")
        gpscmd.cmd = ffi.C.GPS_GET_CABLE_SENSE
        return self:send_command_buffer(gpscmd)
    end
end

return gps
