require('lj_include'){
    'stdio',
    'stdlib',
    'string',
    'unistd',
}

local log = require 'log'
log:open('camera_test', log.USER)

if not arg[1] or not arg[2] or not arg[3] then
    io.write("USAGE: luajit camera_test.lua vendor model serial host" ..
             " firmware rate,resolution,compress [save]\n" ..
             " - vendor: Camera Maker          ex: AXIS, Vivotek\n" ..
             " - model: Camera Model           ex: P1311, IP8332C\n" ..
             " - serial: Serial Number         ex: 00408cb0c2e1\n" ..
             " - host: IP address or DNS name  ex: 172.26.144.85\n" ..
             " - firmware: Firmware            ex: 5.40.9.2\n" ..
             " - rate,resolution,compress:     ex: 30,640x480,30\n" ..
             " - save: Create save # of images in /tmp [1 - 100]\n")
    io.write("\n\tOR\n\n")
    io.write("USAGE: luajit camera_test.lua vendor api <method-name> [options]\n\n")
    os.exit(1)
end

local ev = require 'ev'
local net = require 'net_ffi'
local ffi = require 'ffi'
local C = ffi.C

-- GNU C Library 'libc'

ffi.cdef[[
/* Pinger stuff */
struct iphdr {
    uint8_t       ihl_version;
    uint8_t       tos;
    uint16_t      tot_len;
    uint16_t      id;
    uint16_t      frag_off;
    uint8_t       ttl;
    uint8_t       protocol;
    uint16_t      check;
    uint32_t      saddr;
    uint32_t      daddr;
} __attribute__((packed))__;

struct icmphdr {
    uint8_t       type;
    uint8_t       code;
    uint16_t      checksum;
    union {
        struct {
            uint16_t      id;
            uint16_t      sequence;
        } echo;
        uint32_t   gateway;
        struct {
            uint16_t      __unused;
            uint16_t      mtu;
        } frag;
    } un;
    uint8_t       data[56];
} __attribute__((packed))__;

enum {
    ICMP_ECHO = 8,
};
]]

-- camera implementations

local camera = {
    ['axis'] = require 'axis_camera',
    ['vivotek'] = require 'vivotek_camera'
}

local vendor = arg[1]:lower() or 'axis'

-- Program Starts Here

if arg[2]:lower() == 'api' then
    if not arg[1] or not arg[2] or not arg[3] then
        io.write("USAGE: luajit camera_test.lua vendor api <method-name> [options]\n")
        os.exit(1)
    end
    if arg[3] == 'probe' then
        -- Vendor specific probe method
        camera[vendor]:probe()
    end
    if arg[3] == 'discover' then
        local ethint = '127.0.0.1'
        -- Get 'ethint' IP Zeroconf address
        local function get_zcip_addr()
            local addrs = net.getifaddrs('ethint')
            for _, addr in ipairs(addrs) do
                if addr:find('^169%.254%.') then
                    return addr
                end
            end
        end
        local ll = ev.Loop.new()
        ev.Timer.new(function(l, t, e)
                ethint = get_zcip_addr()
                if ethint then
                    l:unloop()
                end
            end, 1, 1):start(ll)
        -- Wait here for ZCIP assigned address to show up!
        ll:loop()
        local iface = ffi.new('char [7]', 'ethint')
        local ptr = ffi.new('char *')
        local line = ffi.new('char [81]')
        local ip = ffi.new('char [16]')
        local mac = ffi.new('char [18]')
        local serial = ffi.new('char [13]')
        local arp_list = {}
        -- Vendor specific probe method
        camera[vendor]:probe()
        local arp = ffi.new('FILE *', C.fopen('/proc/net/arp', 'r'))
        if arp == nil then
            log:error(__"camera_test: fopen {0}",
                              ffi.string(C.strerror(ffi.errno())))
        end
        while C.fgets(line, ffi.sizeof(line), arp) ~= nil do
            if C.strstr(line, iface) ~= nil then
                C.sscanf(line, "%17[^ ]", ip)
                ptr = C.strstr(line, ":")
                ptr = ptr - 2
                C.sscanf(ptr, "%2s:%2s:%2s:%2s:%2s:%3[^ ]",
                         serial,
                         serial + 2,
                         serial + 4,
                         serial + 6,
                         serial + 8,
                         serial + 10)
                C.sscanf(ptr, "%18[^ ]", mac)
                if net.Pinger:ping(ffi.string(ip), 3, 'ethint',
                                   ev.Loop.default, ethint) then
                    table.insert(arp_list,{
                                 ip = ffi.string(ip),
                                 serial = ffi.string(serial)
                             })
                end
            end
        end
        -- Vendor specific discover method
        local o = camera[vendor]:discover(arp_list)
        for i = 1, #o do
            print(o[i].vendor .. "-" ..
                  o[i].model .. "-" ..
                  o[i].serial .. "-" ..
                  o[i].ip .. "-" ..
                  o[i].firmware)
        end
    end
else
    if not arg[1] or not arg[2] or not arg[3] or not arg[4] or not arg[5] then
        io.write("USAGE: luajit camera_test.lua vendor model serial host" ..
                 " firmware rate,resolution,compress [save]\n" ..
                 " - vendor: Camera Maker          ex: AXIS, Vivotek\n" ..
                 " - model: Camera Model           ex: P1311, IP8332C\n" ..
                 " - serial: Serial Number         ex: 00408cb0c2e1\n" ..
                 " - host: IP address or DNS name  ex: 172.26.144.85\n" ..
                 " - firmware: Firmware            ex: 5.40.9.2\n" ..
                 " - rate,resolution,compress:     ex: 30,640x480,30\n" ..
                 " - save: Create save # of images in /tmp [1 - 100]\n")
        os.exit(1)
    end

    local settings = ffi.new('char [17]', arg[6] or '30,640x480,30')
    local images = arg[7] or '0'
    local save = tonumber(images)
    if save > 100 then
        save = 100
    end
    if save < 0 then
        save = 0
    end
    local finished = false

    -- setup camera

    camera[vendor]:setup(vendor, arg[2], arg[3], arg[4], arg[5])

    if save > 0 then
        local frame_rate = ffi.new('char [3]')
        local image_size = ffi.new('char [10]')
        local image_compression = ffi.new('char [4]')

        local ptr = ffi.new('char *')
        C.sscanf(settings, "%2[^,]s", frame_rate)
        ptr = C.strstr(settings, ',') + 1
        C.sscanf(ptr, "%9[^,]s", image_size)
        ptr = C.strstr(ptr, ',') + 1
        C.sscanf(ptr, "%s", image_compression)

        if C.atoi(frame_rate) > 30 then
            frame_rate = ffi.new('char [3]', '30')
        end

        if C.atoi(image_compression) > 100 then
            image_compression = ffi.new('char [4]', '100')
        end

        camera[vendor]:spec_frame_rate(ffi.string(frame_rate))
        camera[vendor]:spec_image_size(ffi.string(image_size))
        camera[vendor]:spec_image_compression(ffi.string(image_compression))
    end

    local image_list = {}

    if save > 0 then
        for i = 1, save do
            local fn = ffi.new('unsigned char[100]')
            local fp = ffi.new('FILE *')
            C.sprintf(fn, "/tmp/image%04d.jpg", ffi.new('int', i))
            fp = C.fopen(fn, 'w')
            table.insert(image_list, { fp = fp })
            C.usleep(10000)
        end
    end

    local function close_image_list()
        if save > 0 then
            for i = 1, tonumber(save) do
                C.fclose(image_list[i].fp)
            end
        end
    end

    local function image_write(buf, len)
        local count = camera[vendor]:frame_count() + 1
        if save > 0 then
            if count > save then
                finished = true
                return
            end
            if image_list[count].fp then
                C.fwrite(buf, 1, len, image_list[count].fp)
            end
        end
        io.write("Frame Count = " .. tonumber(count) .. "\n" ..
                 "Frame Size  = " .. tonumber(len) .. "\n")
        camera[vendor]:frame_count(count)
    end

    camera[vendor]:channel_write(ffi.cast('void *(*)(void *, size_t)', image_write))

    -- start video streaming

    io.write("camera_test: VIDEO_CONNECT\n")

    if not camera[vendor]:start('mjpeg') then
        if save > 0 then
            for i = 1, tonumber(save) do
                C.fclose(image_list[i].fp)
            end
        end
        os.exit(1)
    end

    io.write("camera_test: VIDEO_STREAMING\n")

    -- monitor/consume video stream

    while not finished do
        C.usleep(100000)
    end

    -- video streaming is done

    camera[vendor]:stop()

    if save > 0 then
        for i = 1, save do
            C.fflush(image_list[i].fp)
            C.fclose(image_list[i].fp)
        end
    end
end

io.write("camera_test: Done.\n")
