

local aux = require 'aux'
local can = require 'can'
local ffi = require 'ffi'
local listener = require 'listener'
local dbus = require 'dbus'
local sample_clock = require 'sample_clock'
local bus = dbus.bus_get('system')
local timing = assert(bus:auto_proxy('com.somat.armada.Timing',
                                     '/com/somat/armada/Timing'))

local clkfreq = sample_clock.master_rate

local cmd_timeout = 0.01


local function display_vbm_message(pktstruct)
    if pktstruct.is_control_frame == 1 then
        print(ffi.string(pktstruct.msg, pktstruct.msg_length))
    else
        local s = ffi.string(pktstruct.msg, pktstruct.msg_length)
        local bytes = {s:byte(1,-1)}
        for i=1,#bytes do io.write(string.format("%02X ", bytes[i])) end
        print("TS "..tonumber(pktstruct.ts))
    end
end


local function display_can_message(pktstruct)
    local ff = pktstruct.frame_format
    local ecc = pktstruct.error_capture_code
    local rxe = pktstruct.rx_error_count
    local txe = pktstruct.tx_error_count
    local fmt = "CAN %s => FF=%02X ECC=%02X RXE=%02X TXE=%02X "
    local ext_frame = false
    if ff > 127 then
        ff = ff - 128
        ext_frame = true
    end
    local rtr = false
    if ff > 63 then
        ff = ff - 64
        rtr = true
    end
    local direction = "RX"
    if ff > 15 then
        ff = ff - 16
        direction = "TX"
    end
    io.write(string.format(fmt, direction, ff, ecc, rxe, txe))
    print("TS "..tonumber(pktstruct.ts))
    if ext_frame then
        io.write(string.format("          ID=%08X MSG=", pktstruct.can_id))
    else
        io.write(string.format("          ID=%04X MSG=", pktstruct.can_id))
    end
    for di=1,ff do
        io.write(string.format("%02X ", pktstruct.can_message[di - 1]))
    end
    print("")
end


local function process_elements(channel_obj, name, element)
    local msglog = channel_obj.msglog
    if not msglog then
        channel_obj.msglog = {}
        msglog = channel_obj.msglog
    end
    local palog = msglog[name]
    if not palog then
        msglog[name] = {}
        palog = msglog[name]
    end
    palog[#palog + 1] = element
end


local function process_heartbeats(channel_obj, name, ts)
    local msglog = channel_obj.msglog
    if not msglog then
        channel_obj.msglog = {}
        msglog = channel_obj.msglog
    end
    local palog = msglog[name]
    if not palog then
        msglog[name] = {}
        palog = msglog[name]
    end
    local hblog = palog.hb
    if not hblog then
        palog.hb = {}
        hblog = palog.hb
    end
    hblog[#hblog + 1] = ts
end


local function loopback_test(can_devname, aux_devname)
    -- Setup CAN
    local can_channel = can:new(can_devname, "can.bin", clkfreq)
    can_channel.element_action_func = process_elements
    can_channel.heartbeat_func = process_heartbeats
    can_channel:bus_setup({baud_rate = 250000, sjw = 2, tseg1 = 13, tseg2 = 2 })
    assert(listener({can_channel}, cmd_timeout))
    can_channel:enable_terminator()
    assert(listener({can_channel}, cmd_timeout))
    can_channel:get_current_ts()
    local msg = assert(listener({can_channel}, cmd_timeout))
    local can_start_time = msg.ts + clkfreq
    can_channel:rx_start(can_start_time - 3, 1, 16, 128)
    assert(listener({can_channel}, cmd_timeout))
    can_channel:tx_start_buf(0, 16, 128)
    assert(listener({can_channel}, cmd_timeout))

    -- Setup VBM
    local aux_channel = aux:new(aux_devname, "aux.bin", clkfreq)
    aux_channel.element_action_func = process_elements
    aux_channel.heartbeat_func = process_heartbeats
    aux_channel:setup_vbm()
    assert(listener({aux_channel}, cmd_timeout))
    aux_channel:txdr_power_on()
    assert(listener({aux_channel}, cmd_timeout))
    aux_channel:get_current_ts()
    local msg = assert(listener({aux_channel}, cmd_timeout))
    local aux_start_time = msg.ts + clkfreq
    aux_channel:rx_start(aux_start_time - 3, 1, 16, 128)
    assert(listener({aux_channel}, cmd_timeout))
    aux_channel:event_start(aux_start_time - 3, 1, 16, 128)
    assert(listener({aux_channel}, cmd_timeout))
    aux_channel:tx_start(0, 16, 128)
    assert(listener({aux_channel}, cmd_timeout))
    aux_channel:tx_cmd_msg(0, {"HIGH_SPEED=YES",
                               "TERMINATION_ENABLE=YES",
                               "CAN_MODE=INITIALIZE",
                               "SYNC_JUMP_WIDTH=3",
                               "BAUD_RATE_PRESCALER=4",
                               "TIME_SEGMENT_1=11",
                               "TIME_SEGMENT_2=4",
                               "SAMPLES_PER_BIT=3",
                               "LISTEN_ONLY_MODE=NO",
                               "CAN_MODE=NORMAL",
                               "APPSTATE?",
                               "RUN=YES"
                               })
    assert(listener({aux_channel}, 3.0, 12))    -- Must receive 12 RX msgs

    local msgs = can_channel:allocate_msg_array(1)
    can_channel:insert_msg(msgs, 1, 0, 0x93426665, 'Hi There')
    can_channel:tx_append_msgs(0, msgs)

    aux_channel:get_current_ts()
    msg = assert(listener({aux_channel}, cmd_timeout))
    local vmsgs = aux_channel:allocate_msg_array(2)
    local vbm_msg_1 = string.char(0x49, 0x93, 0x42, 0x66, 0x67)..'Hello'
    local vbm_msg_2 = string.char(0x89, 0x93, 0x42, 0x66, 0x68)..'World'
    aux_channel:insert_msg(vmsgs, 1, 0, vbm_msg_1)
    aux_channel:insert_msg(vmsgs, 2, msg.ts + clkfreq, vbm_msg_2)
    aux_channel:tx_append_msgs(0, vmsgs)


    assert(listener({aux_channel, can_channel}, 3.0))

    print("Cleaning up")

    aux_channel:tx_stop(0)
    assert(listener({aux_channel}, cmd_timeout))
    can_channel:tx_stop(0)
    assert(listener({can_channel}, cmd_timeout))
    aux_channel:rx_stop()
    assert(listener({aux_channel}, cmd_timeout))
    can_channel:rx_stop()
    assert(listener({can_channel}, cmd_timeout))

    aux_channel:txdr_power_off()
    assert(listener({aux_channel}, cmd_timeout))

    aux_channel:halt()
    can_channel:halt()

    local dump_log = function(mlog, id, show_heartbeats, procfunc)
        print("Dumping "..id.." Logs")
        for k,v in pairs(mlog) do
            if type(k) == "number" then
                io.write('MSG: ')
                local bytes = {v:byte(1,-1)}
                for i=1,#bytes do io.write(string.format("%02X ", bytes[i])) end
                print("")
            else
                print("Dumping "..k)
                for k2,v2 in pairs(v) do
                    if k2 == 'hb' then
                        if show_heartbeats then
                            print("Heartbeats")
                            for k3,v3 in pairs(v2) do
                                print(k3, v3)
                            end
                        end
                    else
                        procfunc(v2)
                    end
                end
            end
        end
    end

    dump_log(aux_channel.msglog, 'VBM', false, display_vbm_message)
    print("Sample Total = "..aux_channel.sample_count)
    dump_log(can_channel.msglog, 'CAN', false, display_can_message)
    print("Sample Total = "..can_channel.sample_count)
    print("Done")
    os.exit(0)
end




if #arg < 2 then
    print("Usage can_vbm_test <can device name> <aux device name>")
    os.exit(1)
end


loopback_test(arg[1], arg[2])

