local ffi = require 'ffi'

ffi.cdef[[
/*
 * NOTE - All of this stuff MUST match what is defined in header files for the
 * QuantumX framework DLL!
 */
typedef unsigned char           uint8_t;
typedef unsigned short          uint16_t;
typedef unsigned long           uint32_t;
typedef unsigned long long      uint64_t;
typedef char                    int8_t;
typedef short                   int16_t;
typedef long                    int32_t;
typedef long long               int64_t;

typedef uint16_t                t_adr_res_type;
typedef uint8_t                 t_tel;
typedef uint16_t                t_trans_type;

typedef uint64_t                tHBM_UUID;

struct address_resolution_hdr {
    t_adr_res_type      type;   // One of types below
    uint16_t            label;  // Essentially a transaction ID (AFAIKT)
} __attribute__((packed));

struct communications_hdr {
    uint32_t            start;          // Always 0x12345678
    uint8_t             len;            // Size of this header
    t_tel               next_hdr_type;  // Type of next header
    uint16_t            payload_length; // Sizeof payload (including next hdr)
    tHBM_UUID           uuid;           // UUID of source
} __attribute__((packed));

struct communications_tail {
    int32_t             tail;           // Always 0x87654321
} __attribute__((packed));

struct transaction_hdr {
    t_trans_type        type;           // Transaction type (see below)
    uint16_t            label;          // Essentially a transaction id (AFAIKT)
    tHBM_UUID           uuid;           // UUID of destination
} __attribute__((packed));

struct transaction_resp_hdr {
    uint16_t            dunno[2];       // Two words I don't know the meaning
    uint32_t            error_code;     // Transaction error status (see below)
} __attribute__((packed));

struct segment_init_hdr {
    uint32_t            total_data;     // Total data expected to rx in segs
} __attribute__((packed));

struct segment_init {
    t_trans_type        type;           // Transaction type (see below)
    uint16_t            label;          // Essentially a transaction id (AFAIKT)
    tHBM_UUID           uuid;           // UUID of destination
    uint32_t            total_data;     // Total data expected to rx in segs
} __attribute__((packed));

struct transaction {
    struct transaction_hdr      hdr;    // Header
    uint16_t                    tr_type[2];     // Transaction type
} __attribute__((packed));

// Some transaction responses DO not include the error code, some do.  Yet
// another wacky feature of the QX device!
struct transaction_resp_short {
    t_trans_type        type;
    uint16_t            label;
    tHBM_UUID           uuid;
    uint16_t            tr_type[2];
} __attribute__((packed));

struct transaction_resp_long {
    t_trans_type        type;
    uint16_t            label;
    tHBM_UUID           uuid;
    uint16_t            tr_type[2];
    uint32_t            error_code;
} __attribute__((packed));

struct segment_hdr {
    t_trans_type        type;           // Transaction type (see below)
    uint16_t            label;          // Essentially a transaction id (AFAIKT)
    tHBM_UUID           uuid;           // UUID of destination
    uint16_t            tr_type[2];     // Transaction type
    uint32_t            error_code;     // Transaction error status (see below)
} __attribute__((packed));

struct subsequent_segment_hdr {
    t_trans_type        type;           // Transaction type (see below)
    uint16_t            label;          // Essentially a transaction id (AFAIKT)
    tHBM_UUID           uuid;           // UUID of destination
} __attribute__((packed));

struct address_response_hdr {
    t_trans_type        type;
    uint16_t            label;
    uint32_t            unknown;
    tHBM_UUID           uuid;
} __attribute__((packed));

enum {
    SEGMENTING_BUF_SIZE = (3<<17),
    MAX_STREAM_BUF_SIZE = (1<<16),
};

struct sbuf {
    uint8_t     buf[SEGMENTING_BUF_SIZE];
    uint32_t    offset;
};


/*
 * Values for start in comm header and tail in tail header
 */
enum {
    START_OF_MESSAGE = 0x12345678,
    END_OF_MESSAGE = 0x87654321
};

/*
 * Values for type in address_resolution_hdr above
 */
enum {
    ADR_REQUEST_INIT = (t_adr_res_type)1,
    ADR_REQUEST_DEVICE_TYPE = (t_adr_res_type)3,
    ADR_RESPONSE = (t_adr_res_type)0x0100,
    ADR_RESPONSE_PENDING = (t_adr_res_type)0x0101,
};

typedef uint32_t        tDeviceType;

/*
 * Format of payload accompanying address response header
 */
struct adr_response_payload {
    tDeviceType type;
    tHBM_UUID   uuid;
} __attribute__((packed));

/*
 * Values for type in adr_respose_payload above
 */
enum {
    DEVTYPE_ROUTER = 0x00000001,
    DEVTYPE_LEAF = 0x00000002,
    DEVTYPE_VIAROUTER_FLAG = 0x80000000,
    DEVTYPE_ROUTER_VIAROUTER = (DEVTYPE_ROUTER | DEVTYPE_VIAROUTER_FLAG),
    DEVTYPE_LEAF_VIAROUTER = (DEVTYPE_LEAF | DEVTYPE_VIAROUTER_FLAG),
    DEVTYPE_UNKNPOWN
};

/*
 * Values for next_hdr_type in communications_hdr above
 */
enum {
    NEXT_HDR_TYPE_NONE = (t_tel)0,
    NEXT_HDR_TYPE_TRANSACTION = (t_tel)1,
    NEXT_HDR_TYPE_ADDRESS_RESOLUTION = (t_tel)2,
    NEXT_HDR_TYPE_ADR_DEVICE_TYPE = (t_tel)3
};

/*
 * Values for type in transaction_hdr above
 */
enum {
    TRANSACTION_TYPE_REQUEST = (t_trans_type)1,
    TRANSACTION_TYPE_RESPONSE = (t_trans_type)2,
    TRANSACTION_TYPE_RESPONSE_PENDING = (t_trans_type)3,
    TRANSACTION_TYPE_REQUEST_SEGMENT_INIT = (t_trans_type)0x10,
    TRANSACTION_TYPE_REQUEST_SEGMENT = (t_trans_type)0x11,
    TRANSACTION_TYPE_REQUEST_SEGMENT_ACK = (t_trans_type)0x12,
    TRANSACTION_TYPE_RESPONSE_SEGMENT_INIT = (t_trans_type)0x13,
    TRANSACTION_TYPE_RESPONSE_SEGMENT = (t_trans_type)0x14,
    TRANSACTION_TYPE_RESPONSE_SEGMENT_ACK = (t_trans_type)0x15,
    TRANSACTION_TYPE_RESPONSE_SEGMENT_PENDING = (t_trans_type)0x16,
};

struct transaction_req {
    uint16_t    data[2];
} __attribute__((packed));

enum {
    TR_DUNNO_0 = 1,
    TR_XML = TR_DUNNO_0,
    TR_GET_DATA = TR_XML,
    TR_CHANNEL_DATA = TR_XML,
    TR_CHANNEL_SUBSCRIBE = 2,
    TR_SCHEMA = 3,
    TR_CHANNEL_REQUEST = 5,
    TR_CHANNEL_LIST = 6,
    TR_SET_CHANNEL_PARAMETER = 9,
    TR_DUNNO_1 = 0xb,
    TR_READ_TEDS_BINARY = 0xd,
    TR_WRITE_TEDS_BINARY = 0xe,
    TR_PLAIN_XML = TR_CHANNEL_SUBSCRIBE,
};

/*
 * Values for session state
 */
enum {
    SESSION_STATE_INVALID,
    SESSION_STATE_INITIALIZING,

    SESSION_STATE_GET_CURRENT_PARAMETERS,
    SESSION_STATE_GET_CHANNEL_LIST,
    SESSION_STATE_SUBSCRIBE_CHANNEL,
    SESSION_STATE_DATA_TRANSFER,
    SESSION_STATE_READY,
};

/*
 * Taken from Darmstadt source file
 */
enum tError
{
	/// unspecified error
	ERR_NO_SUCCESS                                      =      -1,
	/// well, it worked
	ERR_SUCCESS                                         =       0,
	/// unspecified warning
	WARN_UNSPECIFIED                                    =    1000,
	/// value was adopted
	WARN_VALUE_FIT                                      =       9,


	// **********************************************
	// do not use, because used for texts:              80 ... 1000
	// **********************************************

	// **********************************************
	// CX27:
	// **********************************************

	ERR_FIELDBUS_INIT_DRIVER                            = -1000,
	ERR_FIELDBUS_ACCESS_OD                              = -1001,
	ERR_FIELDBUS_READ_ODXML                             = -1002,
	ERR_FIELDBUS_START_OD                               = -1005,
	ERR_FIELDBUS_STOP_OD                                = -1006,
	ERR_FIELDBUS                                        = -1010,
	ERR_FIELDBUS_1394ISO_INIT                           = -1100,
	ERR_FIELDBUS_1394ISO_START                          = -1101,
	ERR_FIELDBUS_1394ISO_STOP                           = -1102,
	ERR_FIELDBUS_1394ISO_RELEASE                        = -1103,
	ERR_FIELDBUS_MULTIISOINTERFACES                     = -1104,
	ERR_FIELDBUS_INVALIDPARID                           = -1105,
	ERR_FIELDBUS_STATECHANGE                            = -1106,
	ERR_FIELDBUS_INVALIDTIME                            = -1107,
	ERR_FIELDBUS_ACCESS_SM                              = -1108,
	ERR_FIELDBUS_UNAVAILABLPAR                          = -1109,

	ERR_PARAM_INVALID_IOMODE                            = -1200,
	ERR_PARAM_INVALID_SOURCE                            = -1202,

	//                                                    -9999

	// **********************************************
	// do not use, because used for texts:              90220 ... 90500
	// **********************************************

	// **********************************************
	// Achtung! Keine neuen Texte im Bereich 100000 bis 102000 bzw. -100000 bis -102000, da diese bereits durch Texte fuer Parameterbaeume vergeben sind!
	// Attention! Do not add text between 100000 through 102000 or -100000 through -102000, since these areas are reserved for texts of parameter trees!
	// **********************************************

	ERR_SCALING_NOT_SUPPORTED_FOR_SENSORTYPE            = -100103,
	ERR_SCALING_INVALID_TYPE                            = -100104,
	ERR_SCALING_STRAINGAGE_NOT_SUPPORTED                = -100105,
	ERR_RESTORING_FACTORY_SCALING                       = -100106,
	ERR_SCALING_DIV_BY_ZERO                             = -100110,
	ERR_SCALING_NUMBER_OF_SEGMENTS                      = -100112,

	ERR_FILTERTYPE_NOT_SUPPORTED                        = -100201,
	ERR_FILTERCHARACT_NOT_SUPPORTED                     = -100202,
	ERR_FILTERFREQ_NOT_SUPPORTED                        = -100203,

	ERR_CONNMODE_NOT_SUPPORTED                          = -100450,

	// Basic sensor
	ERR_SENSORTYPE_NOT_SUPPORTED                        = -100506,
	ERR_MAXELECVAL_NOT_DEFINED                          = -100507,
	ERR_OUTPUTRATE_VS_FILTER_NOT_SUPPORTED              = -100510,
	ERR_OUTPUTRATE_NOT_SUPPORTED                        = -100511,

	// Voltage input
	ERR_VOLTAGERANGE_NOT_SUPP                           = -100700,

	// Current input
	ERR_CURRENTRANGE_NOT_SUPP                           = -100710,

	// Bridge sensor
	ERR_BRIDGETYPE_NOT_SUPPORTED                        = -100800,
	ERR_WIRING_NOT_SUPPORTED                            = -100802,
	ERR_WIRING_NOT_SUPPORTED_CHECK_HW_REVISION          = -100803,
	ERR_EXCITFREQ_NOT_SUPPORTED                         = -100804,
	ERR_EXCITAMPL_NOT_SUPPORTED                         = -100806,
	ERR_IMPEDANCE_NOT_SUPPORTED                         = -100807,

	// Pt Sensor
	ERR_PT_RESISTANCE_NOT_SUPPORTED                     = -100850,

	// Thermocouple
	ERR_TC_TYPE_NOT_SUPPORTED                           = -100860,
	ERR_ONLY_PHYSUNIT_CELSIUS_SUPPORTED                 = -100861,

	// Poti Sensor
	ERR_RESISTANCE_MAX                                  = -100900,
	ERR_RESISTANCE_MIN                                  = -100901,

	// Resistor
	ERR_RESISTOR_NOT_SUPPORTED                          = -100904,

	// SSI coding
	ERR_SSI_CODING_NOT_SUPPORTED                        = -101002,

	// Pressure sensor
	ERR_PRESSURE_RANGE_NOT_SUPPORTED                    = -101010,
	ERR_PRESSURE_MAX_AVAILABLE                          = -101011,
	ERR_PRESSURE_TYPE_ONLY_ABSOLUTE_SUPPORTED           = -101012,
	ERR_PRESSURE_TYPE_ONLY_RELATIVE_SUPPORTED           = -101013,
	ERR_PRESSURE_TYPE_NOT_SUPPORTED                     = -101014,
	ERR_PRESSURE_SENSOR_NOT_AVAILABLE                   = -101015,
	ERR_PRESSURE_MIN_AVAILABLE                          = -101016,

	// frequency input
	ERR_FREQUENCYRANGE_NOT_SUPPORTED                    = -101020,

	// Sensor supply not supported
	ERR_SENSORSUPPLY_NOT_SUPPORTED                      = -101100,

	// Zero balance not allowed/inhibited
	ERR_SETZERO_INHIBITED                               = -101110,

	// Analog output
	ERR_ANALOG_OUT_SCALING_NOT_SUPPORTED                = -101201,
	ERR_ANALOG_OUT_SETZERO                              = -101202,

	// Analog input parameter
	ERR_HWSETUP_BRIDGE_NOSETTING                        = -102020,
	ERR_HWSETUP_VOLTAGE_NOSETTING                       = -102021,
	ERR_HWSETUP_CURRENT_NOSETTING                       = -102022,
	ERR_HWSETUP_TC_NOSETTING                            = -102023,
	ERR_HWSETUP_PT_NOSETTING                            = -102024,
	ERR_HWSETUP_POTI_NOSETTING                          = -102025,
	ERR_HWSETUP_SSI_NOSETTING                           = -102026,
	ERR_HWSETUP_COUNTER_NOSETTING                       = -102027,
	ERR_HWSETUP_FREQ_NOSETTING                          = -102028,
	ERR_HWSETUP_INDBRIDGE_NOSETTING                     = -102029,
	ERR_HWSETUP_LVDT_NOSETTING                          = -102030,
	ERR_HWSETUP_PRESSURE_NOSETTING                      = -102031,
	ERR_HWSETUP_IEPE_NOSETTING                          = -102032,
	ERR_HWSETUP_PWM_NOSETTING                           = -102033,
	ERR_HWSETUP_PULSEWIDTH_NOSETTING                    = -102034,
	ERR_HWSETUP_PERIOD_NOSETTING                        = -102035,
	ERR_HWSETUP_RESISTOR_NOSETTING                      = -102036,
	ERR_HWSETUP_IRIG_FORMAT_NOT_SUPPORTED_BY_CONNECTOR  = -102037,

	WARN_CONNECTOR_SET_TO_VOLTAGE_BECAUSE_ANOTHER_SET_TO_IRIG = 102038,

	ERR_PREVIOUS_SETTING_RESTORED                       = -104010,

	WARN_SCALING_NR_OF_SEGMENTS_CHANGED                 =  104100,
	WARN_SCALING_NR_OF_COEFFS_CHANGED                   =  104101,

	WARN_EXTERNAL_ADJUST_COEFFS_NOT_AVAILABLE           =  104110,
	ERR_EXTERNAL_ADJUST_COEFFS_NOT_AVAILABLE            = -104110,

	ERR_OUTPUTUNIT_INCOMPATIBLE_TO_SENSORUNIT           = -104112,
	WARN_OUTPUTUNIT_INCOMPATIBLE_TO_SENSORUNIT          =  104112,
	WARN_OUTPUTUNIT_ONLY_FIRST_SIGNAL                   =  104113,

	ERR_AUTOCAL_REPTIME_NOT_SUPPORTED                   = -104200,
	ERR_AUTOCAL_SENSOR_NOT_SUPPORTED                    = -104201,
	WARN_AUTOCAL_NOT_SUPPORTED                          =  104202,
	ERR_AUTOCAL_DISABLED                                = -104203,
	ERR_AUTOCAL_SENSOR_NOT_CONNECTED                    = -104204,
	ERR_AUTOCAL_NOT_SUPPORTED                           = -104205,


	// **********************************************
	// TEDS translator
	// **********************************************

	// kein Sensor in der TEDS Beschreibung gefunden
	ERR_TEDS_NOSENSOR                                   = -105000,
	// Fehler beim Auswerten des Teds
	ERR_TEDS_NO_DATA_OR_INVALID                         = -105001,
	// Keine MIN Messwert in phys. Einheit
	ERR_TEDS_NOMINPHYSVAL                               = -105002,
	// Keine MAX Messwert in phys. Einheit
	ERR_TEDS_NOMAXPHYSVAL                               = -105003,
	// Keine MIN Messwert in elek. Einheit
	ERR_TEDS_NOMINELECVAL                               = -105004,
	// Keine MIN Messwert in elek. Einheit
	ERR_TEDS_NOMAXELECVAL                               = -105005,
	// Keine Messstellenkennung
	ERR_TEDS_NOMEASID                                   = -105006,
	// kein HBM UCC definiert
	ERR_TEDS_NOMDEF_UCC                                 = -105007,
	// keine Kalibrierinformation
	ERR_TEDS_NOCALINITIALS                              = -105008,
	// kein HBM User Defined Id String definiert
	ERR_TEDS_NOMDEF_USERDEFID                           = -105009,
	// Kein TEDS Binaerblock vorhanden
	ERR_TEDS_NODATA                                     = -105010,
	// Fehler in einem XPATh oder nicht vorhanden
	ERR_TEDS_NOXPATH                                    = -105012,
	// Analysieren des TEDS Bin blocks fehlgeschlagen
	ERR_TEDS_NOPARSEBIN                                 = -105014,
	// Sensorparametrierung schlug
	ERR_TEDS_NOADJUSTTEDSSET                            = -105016,
	// Kein Verstaerker-HAL zum Kanal gefunden
	ERR_TEDS_NOAMP                                      = -105018,
	// Kein Nominale Amplitude der Bruecken-Speisespannung
	ERR_TEDS_NOEXCITEAMPLNOM                            = -105020,
	// Kein minimale Amplitude der Bruecken-Speisespannung
	ERR_TEDS_NOEXCITEAMPLMIN                            = -105022,
	// Kein maximale Amplitude der Bruecken-Speisespannung
	ERR_TEDS_NOEXCITEAMPLMAX                            = -105024,
	// keine Impedanz
	ERR_TEDS_SENSORIMPED                                = -105030,
	// kein k-Faktor
	ERR_TEDS_GAUGEFACTOR                                = -105031,
	// kein Brueckenschaltungstyp
	ERR_TEDS_BRIDGETYPE                                 = -105032,
	ERR_TEDS_TC_TYPE                                    = -105034,
	ERR_TEDS_PT_TYPE                                    = -105036,
	ERR_TEDS_LOWPASSFILTER                              = -105038,
	ERR_TEDS_ZEROSPANSCALING                            = -105040,
	ERR_TEDS_GAINSCALING                                = -105041,
	// allgemeiner Fehler beim TEDS-Parametrieren
	ERR_TEDS_PARAM                                      = -105050,
	// TEDS data not supported by connector
	ERR_TEDS_NOT_SUPPORTED_BY_CONNECTOR                 = -105051,

	// Keine Caltable
	ERR_TEDS_NOCALTABLE_DOMAIN                          = -105054,
	// Kein Power of domain value
	ERR_TEDS_NOCALCURVEPOWER                            = -105056,
	// Zu viele Stuetzpunkte
	ERR_TEDS_CALLISTTOLARGE                             = -105060,
	// Buffer for TEDS data too small
	ERR_TEDS_BUFFER_TOO_SMALL                           = -105061,
	// Teds (RFID) darf nicht geschrieben werden
	ERR_TEDS_WRITE_NOT_ALLOWED                          = -105062,
	// TEDS TID don't match
	ERR_TEDS_TID_DONT_MATCH                             = -105063,
	// Scaling Type wird nicht unterstuetzt
	ERR_TEDS_SCALINGTYPE_NOT_SUPPORTED                  = -105100,


	// **********************************************
	// Speed mode
	// **********************************************

	ERR_SPEEDMODE_CHANGING_NOT_POSSIBLE_SWITCH_OFF      = -105201,
	ERR_SPEEDMODE_TOO_MANY_CONNECTORS_HIGH              = -105202,
	ERR_SPEEDMODE_MIXED_MODE_NOT_POSSIBLE               = -105204,
	WARN_SPEEDMODE_CHANGED_TO_HIGH                      =  105205,
	WARN_HIGHSPEEDMODE_NOT_SUPPORTED                    =  105206,
	ERR_UNKNOWN_SPEEDMODE_SELECTED                      = -105207,
	ERR_SPEEDMODE_HIGH_NOT_SUPPORTED                    = -105208,

	// **********************************************

	ERR_CONNECTORSOURCE                                 = -105210,


	// **********************************************
	// Engineering units
	// **********************************************

	//  physikalische Einheiten
	/// Unit(s) not found
	ERR_PHYSUNIT_NOT_FOUND                              =  -106000,
	/// Unit code not valid
	ERR_PHYSUNIT_ILLEGAL_NUMBER                         =  -106002,
	/// Unit not convertible
	ERR_PHYSUNIT_NOT_RELATED                            =  -106004,
	/// factors are erroneous (i.E. scaling)
	ERR_PHYSUNIT_SCALING_FACTOR_ZERO                    =  -106006,


	// **********************************************
	// Parametrization
	// **********************************************

	WARN_WIRING_ALTERED                                 =  119001,
	WARN_SUPPLY_ALTERED_TO_MAX                          =  119002,
	WARN_EXCITATION_FREQ_ALTERED                        =  119003,
	WARN_EXCITATION_AMPL_ALTERED                        =  119004,

	//SSI Sensor
	WARN_SSI_CLK_FREQ_CHANGED                           =  119005,
	WARN_SSI_RESOLUTION_CHANGED                         =  119006,

	//Filter
	WARN_FILTER_CUTOFF_FREQ_CHANGED                     =  119010,
	WARN_FILTERTYPE_CHANGED                             =  119011,
	WARN_FILTERCHAR_CHANGED                             =  119012,
	WARN_OUTPUTRATE_CHANGED                             =  119015,

	WARN_USERUNIT_NOT_SUPPORTED                         =  119020,

	ERR_GLITCHFILTER_NOT_SUPPORTED                      = -119024,
	WARN_GLITCHFILTER_CHANGED                           =  119025,

	WARN_ZEROING_TARGET_CHANGED                         =  119080,

	//                                                    -199999
	// **********************************************

	// Allgemeiner HAL Fehler
	ERRHAL_ERROR                                        = -120000,
	// Oeffnen des Treibers zur Ansteuerung der MSPs fehlgeschlagen
	ERRHAL_OPENMSPDRV                                   = -120100,
	// Oeffnen des Treibers zur Ansteuerung des CPLDs fehlgeschlagen
	ERRHAL_OPENCPLDDRV                                  = -120110,
	// auslesen der Temperatur (von Kanal) fehlgeschlagen
	ERRHAL_GETAMPTEMP                                   = -120120,
	// ermitteln des Temp Offsets fehlgeschlagen
	ERRHAL_GETAMPTEMPCAL                                = -120121,
	// Fehler beim Initialisieren des I2C Interface (CPLD)
	ERRHAL_INITI2C                                      = -120125,
	// Fehler beim Einschalten des Kanals
	ERRHAL_ENABLECHN                                    = -120130,
	// Fehler beim Einstellen des Messeingangs
	ERRHAL_SETMEASINPUT                                 = -120134,
	// Fehler beim Einstellen des Messsignals
	ERRHAL_SETSENSCIRCUIT                               = -120135,
	// Fehler beim Einstellen des Kalibriersignals
	ERRHAL_SETCALINPUT                                  = -120140,
	// Fehler beim Einstellen der Verstearkung (Kal.Kanal)
	ERRHAL_SETGAINCAL                                   = -120145,
	// Fehler biem Einstellen der Verstaerkung (Messkanal)
	ERRHAL_SETGAINMEAS                                  = -120150,
	// Fehler beim Einstellen der Sensor Supply
	ERRHAL_SET_SENSORSUPPLY                             = -120155,
	// Fehler beim Einstellen der Rueckkopplung der TF-Endstufe
	ERRHAL_SETFEEDBACK                                  = -120159,
	// Fehler beim Einstellen der Brueckenspeisespannung
	ERRHAL_SETEXCIT                                     = -120160,
	// Fehler beim Lesen der Seriennummer (Amp)
	ERRHAL_GETSNAMP                                     = -120165,
	// Fehler beim Schreiben der Seriennummer (Amp)
	ERRHAL_SETSNAMP                                     = -120170,
	// Fehler beim Lesen der Hardwarerevision (Amp)
	ERRHAL_GETHWRAMP                                    = -120175,
	// Fehler beim Schreiben der Hardwarerevision (Amp)
	ERRHAL_SETHWRAMP                                    = -120180,
	// Fehler beim Ermitteln der TID
	ERRHAL_GETTID                                       = -120185,
	// Fehler beim Lesen von TEDS Daten aus der Hardware
	ERRHAL_GETTEDSDATHW                                 = -120189,
	// Fehler beim Lesen von TEDS Daten
	ERRHAL_GETTEDSDAT                                   = -120190,
	// Fehler TEDS Checksummen
	ERRHAL_TEDSCHECKSUM                                 = -120191,
	// Kein TEDS verfuegbar
	ERRHAL_NOTEDS                                       = -120192,
	// TEDS vorhanden, Inhalt aber nur Nullen
	ERRHAL_TEDS_ALLZERO                                 = -120193,
	// Fehler beim einfuegen der Checksumme in den TEDS Bin Block
	ERRHAL_TEDSCHECKSUMINSERT                           = -120194,
	// Fehler beim Schreiben von TEDS Daten
	ERRHAL_SETTEDSDAT                                   = -120195,
	// Fehler beim Setzen des TEDS Interface (1Wire/0Wire)
	ERRHAL_SETTEDSCON                                   = -120196,
	// Daten passen nicht in den TEDS Chip
	ERRHAL_SIZETEDSDAT                                  = -120197,
	// Fehler beim Einstellen der LED (Amp)
	ERRHAL_SETLEDAMP                                    = -120200,
	// Fehler beim Einstellen der System LED (MX1609)
	ERRHAL_SETSYSLEDAMP                                 = -120201,
	// Fehler beim Stellen der Schalter
	ERRHAL_WRSETTING                                    = -120205,
	// Fehler beim Einstellen der Abtatsrate
	ERRHAL_SETADCCLK                                    = -120210,
	// Fehler beim Ermitteln der hoechsten Abtastrate (aller Kanaele)
	ERRHAL_GETMAXSRAMP                                  = -120211,
	//Fehler beim Einstellen der FM Configs
	ERRHAL_SET_FM_CONF                                  = -120212,
	//Fehler beim Einstellen der SSI Configs
	ERRHAL_SET_SSI_CONF                                 = -120213,
	// Fehler beim Ermitteln der MSP Versionsnummer (Verstaerker)
	ERRHAL_GETMSPVERAMP                                 = -120215,
	// Fehler beim Auslesen des MSP Status (Verstaerker)
	ERRHAL_GETMSPSTATAMP                                = -120216,
	// Fehler beim Synchronisieren der ADCs
	ERRHAL_SYNCADC                                      = -120220,
	// Fehler beim Setzen der Kanal Eigenschaften
	ERRHAL_SETCHNPROP                                   = -120225,
	// Fehler beim Lesen des MSP Registers
	ERRHAL_GETMSPREG                                    = -120230,
	// Fehler beim Setzen des MSP Registers
	ERRHAL_SETMSPREG                                    = -120235,
	// Fehler beim Einstellen eines Testsignals (Messkanal)
	ERRHAL_SETTSTSIGMEAS                                = -120240,
	// Fehler beim Einstellen eines Testsignals (Kalkanal)
	ERRHAL_SETTSTSIGCAL                                 = -120245,
	// Fehler beim Einstellen eines Simulationswerts
	ERRHAL_SETSIMUVAL                                   = -120246,
	// Format fuer Simulationswerte nicht unterstuetzt
	ERRHAL_SETSIMUVALFORMAT                             = -120248,
	// Type fuer Simulationswerte nicht unterstuetzt
	ERRHAL_SETSIMUVALTYPE                               = -120249,
	// Fehler bei der Momentaufnahme(Mess/Kal-Wertepaare)
	ERRHAL_GETSNAPSHOT                                  = -120250,
	// Ueberschreitung der maximalen Anzahl an Snapshotdaten
	ERRHAL_SNAPSHOT_MAXDATA                             = -120251,
	// Fehler beim Berechnen der Phasenkorrektur
	ERRHAL_CALCPHASE                                    = -120252,
	// Keine gueltigen SnapShat Daten vorhanden
	ERRHAL_NO_VALID_SNAPSHOTDATA                        = -120253,
	// Fehler beim MSP Firmware Download
	ERRHAL_MSPFWDOWN                                    = -120255,
	// Fehler beim Lesen der Ur-Kalibriedaten aus dem MSP-Flash
	ERRHAL_GETCALDATHW                                  = -120256,
	// Fehler beim Lesen der Ur-Kalibriedaten aus Attribut
	ERRHAL_GETCALDAT                                    = -120257,
	// Fehler beim Schreiben der Ur-Kalibriedaten ins MSP-Flash
	ERRHAL_SETCALDAT                                    = -120258,
	// Fehler beim Lesen der Checksumme fuer die UrKalibrierdaten
	ERRHAL_SETCSURCAL                                   = -120260,
	// Fehler beim Schreiben der Checksumme fuer die UrKalibrierdaten
	ERRHAL_GETCSURCAL                                   = -120265,
	// Prueefsumme des Loaders stimmt nicht
	ERRHAL_LDRCHKSUM                                    = -120266,
	// Fehler beim Markieren des Messwerts als ungueltig
	ERRHAL_SETMEASVALINVAL                              = -120270,
	// Fehler beim Auswaehlen des Connectors innerhalb der FPGA Registerstruktur (MX460)
	ERRHAL_SELFPGACHN                                   = -120272,
	// Der Divisor zur Reduzierung der Datenrate ist zu hoch
	ERRHAL_REDDATRATE                                   = -120274,
	// Ausgangsdatenrate wird nicht vom FPGA unterstuetzt
	ERRHAL_RATE_NOT_SUP_BY_FPGA                         = -120276,
	// Funktionsblock nicht verfuegbar
	ERRHAL_FB_NOT_AVAILABLE                             = -120277,

	ERRHAL_FB_UNKNOWN_PROPERTY                          = -120278,

	// Gewuenschtes (Fpga-) Testsignal wird nicht unterstuetzt
	ERRHAL_FPGATEST_NOT_SUPPORTED                       = -120279,

	// **********************************************

	// Fehler beim Lesen der Plattform Kennung
	ERRHAL_GETHWIDENTPF                                 = -120300,
	// Fehler beim Schreiben der Plattform Kennung
	ERRHAL_SETHWIDENTPF                                 = -120305,
	// Fehler beim Lesen der Hardwarerevision (Pf)
	ERRHAL_GETHWRPF                                     = -120310,
	// Fehler beim Schreiben der Hardwarerevision (Pf)
	ERRHAL_SETHWRPF                                     = -120315,
	// Fehler beim Lesen der im MSP gespeicherten PLL Spannung (Pf)
	ERRHAL_GETPLLVOLTAGE                                = -120317,
	// Fehler beim Speichern der PLL Spannung im MSP (Pf)
	ERRHAL_SETPLLVOLTAGE                                = -120319,
	// Fehler beim Schreiben der MAC Adresse (Pf)
	ERRHAL_SETMACADDR                                   = -120320,
	// Fehler beim Lesen der MAC Adresse (Pf)
	ERRHAL_GETMACADDR                                   = -120325,
	// Fehler beim Einstellen der LED (Pf)
	ERRHAL_SETLEDPF                                     = -120330,
	// Auslesen der Temperatur (von Platform) fehlgeschlagen
	ERRHAL_GETPFTEMP = -120335,
	// Fehler beim Ermitteln der Modulspannung
	ERRHAL_GETMODVOL = -120340,
	// Fehler beim Ermitteln des Modulstroms
	ERRHAL_GETMODCUR = -120345,
	// Fehler beim Ermitteln des eingespeisten Stroms (Versorgung)
	ERRHAL_GETEXTCUR = - 120350,
	// Fehler beim Ermitteln der Slotnummer auf der Backplane (Plattform)
	ERRHAL_GETSLOTNUM = - 120355,
	// Fehler beim Ermitteln des MSP Application Date (Plattform)
	ERRHAL_GETMSPAPPLDATEPF = -120358,
	// Fehler beim Ermitteln des MSP Loader Date (Plattform)
	ERRHAL_GETMSPLOADERDATEPF = -120359,
	// Fehler beim Ermitteln der MSP Versionsnummer (Plattform)
	ERRHAL_GETMSPVERPF = -120360,
	// Fehler beim Lesen des MSP-Status (Plattform)
	ERRHAL_GETMSPSTATPF = -120361,
	// Fehler beim Ermitteln der Kanaele, die sich in der Strombegrenzung fuer die
	// variable Aufnehmerspeisung befinden
	ERRHAL_GETLIMCURR = -120362,
	// Fehler beim Ermitteln der MSP Versionsnummer (Plattform)
	ERRHAL_SETTSTSIGGEN = -120365,
	// Fehler beim Starten des Data Streams
	ERRHAL_STRTDATSTRM = -120367,
	// Fehler beim Stoppen des Data Streams
	ERRHAL_STOPDATSTRM = -120369,
	// Fehler beim Einstellen der IIS Dummydaten
	ERRHAL_IISDUMDAT = -120371,
	// Fehler beim Aktivieren des IIS Interface
	ERRHAL_ENIIS = -120373,
	// Fehler beim Reset des IIS Interface
	ERRHAL_RSTIIS = -120375,
	// Fehler bei der Plausibilitaetspruefung eines Flash Eintrags
	ERRHAL_CHKFLHETRY = -120377,
	// Fehler beim Ermitteln der hoechsten Abtastrate aller Kanaele
	ERRHAL_GETMAXSRPF = -120379,
	// Fehler beim Firmware Update MSP (Plattform)
	ERRHAL_MSPFWDWNPF = -120385,
	// Fehler beim Einschalten der Kanaele ueber die Plattform
	ERRHAL_ENALLCHNS = -120390,
	// Fehler beim Versuch PSC2 zu aktivieren
	ERRHAL_ENABLEPSC2 = -120395,
	// Fehler beim Versuch PSC2 zu resetten
	ERRHAL_RESETPSC2 = -120396,
	// Fehler beim Versuch PSC2 zu deaktivieren
	ERRHAL_DISABLEPSC2 = -120397,
	// Fehler beim Versuch PSC2 zu initialisieren
	ERRHAL_INITPSC2 = -120398,
	// Fehler beim Versuch Daten aus dem DMA Buffer zu lesen
	ERRHAL_GETDMABUF = -120400,
	// Fehler beim Versuch die DMA Task aufzusetzen und zu starten
	ERRHAL_STARTDMA = -120401,
	// Fehler beim Versuch die DMA Task aufzusetzen und zu starten
	ERRHAL_STOPDMA = -120402,
	// Fehler beim Versuch den I2S Treiber zu oeffnen
	ERRHAL_OPENI2SDRV = -120405,
	// Fehler beim Versuch den I2S Treiber zu schliessen
	ERRHAL_CLOSEI2SDRV = -120406,
	// Fehler beim Versuch die Spg. fuer die CX obere Aufsetzerplatine zu schalten
	ERRHAL_ENCXUPPER = -120408,
	// Fehlerhafte Signalnummer
	ERRHAL_SIGNUMBER = -120410,
	// Fehler in der Parametrierung des FOS Delay (Unbekannte Sample Rate oder falsche FOS Tor Nummer)
	ERRHAL_FOSDELAY = -120412,
	// Parameter zum Skalierung (MX410, MX460) ausserhalb des zulaessigen Bereichs
	ERRHAL_SETSCALPARS = -120414,
	// Unbekannte Signalquelle fuer den DAC
	ERRHAL_DACINPUTTYPE = -120416,
	// Filterausgangsrate kann nicht im FPGA eingestellt werden (Sync Code).
	ERRHAL_UNKNOWNSYNCCODE = -120417,
	// Fehler in der Parametrierung der HBM-Mathefunktionen
	ERRHAL_ADDFUNCMATH = -120418,
	// Unbekannte Matheeinheit
	ERRHAL_UNKNOWN_MATH_TYPE = -120420,
	// Fehlerhafte Ausgangsrate fuer die Spitzenwerteinheit beim MX410
	ERRHAL_PEAK_OUTRATE = -120422,
	// Test fuer Sync Interface (CX) fehlgeschlagen
	ERRHAL_CX_SYNC_TEST = -120424,
	// Nullstellen (der Matheeinheiten) schlug fehl, da noch kein gueltiger Messwert vorhanden war.
	ERRHAL_SETZERO_NO_VALID_MEASVAL = -120426,

	// **********************************************

	// Fehler beim Oeffnen des Treibers
	ERRHAL_FPGAOPENDRV = -120500,
	// Fehler beim Oeffnen des Treibers
	ERRHAL_FPGACLOSEDRV = -120501,
	// Fehler beim Daten/Adressbustest (FPGA)
	ERRHAL_FPGABUSTST = -120502,
	// Fehler beim Lesen vom Treiber (ioctl:read)
	ERRHAL_FPGAREAD = -120503,
	// Fehler beim Schreiben in den Treiber (ioctl:write)
	ERRHAL_FPGAWRITE = -120504,
	// Fehler beim Initialisieren des I2C Interface
	ERRHAL_FPGAINITI2C = -120505,
	// Fehler beim Ruecksetzen des I2C Interface
	ERRHAL_FPGARSTI2C = -120506,
	// Fehler beim Ruecksetzen des HZR
	ERRHAL_FPGARSTHZR = -120507,
	// Fehler beim Ermitteln der FPGA Software Version
	ERRHAL_FPGAGETVER = -120508,
	// Versuch Zugriff auf MSP waehrend Download
	ERRHAL_DOWNLOADMSP = -120509,

	// **********************************************

	// Fehler beim Ermitteln der Temperatur
	ERRHAL_MSPGETTMP = -120700,
	// Fehler beim Messen (Modulspg. , Modulstrom oder ext.Strom)
	ERRHAL_MSPMEAS = -120705,
	// Fehler beim Lesen eines MSP Output Registers
	ERRHAL_MSPRDCTRREG = -120710,
	// Fehler beim Schreiben eines MSP Output Registers
	ERRHAL_MSPWRCTRREG = -120715,
	// Fehler beim Schreiben eines Bits im MSP Output Registers
	ERRHAL_MSPWRREGBIT = -120720,
	// Fehler beim Schreiben des MSP Info Flashs
	ERRHAL_MSPWRINFO = -120725,
	// Fehler beim Lesen vom MSP Info Flash
	ERRHAL_MSPRDINFO = -120730,
	// Fehler beim Einstellen eines Potentiometers (var. Aufnehmerspeisung)
	ERRHAL_MSPSETPOTI = -120740,
	// Fehler beim Auslesen eines Potentiometers (var. Aufnehmerspeisung)
	ERRHAL_MSPGETPOTI = -120745,
	// Fehler beim Ermitteln der Steckplatznummer auf der Backplane
	ERRHAL_MSPGETSLNUM = -120750,
	// Fehler beim Ermitteln der MSP Software Version
	ERRHAL_MSPGETVERSION = -120755,
	// Fehler beim Ermitteln des MSP Status
	ERRHAL_MSPGETSTAT = -120756,
	// Fehler beim Auslesen des Fehlerrgisters (vom MSP)
	ERRHAL_MSPGETERR = -120760,
	// Fehler beim OW Reset (ueber MSP)
	ERRHAL_MSPDOOWRST = -120761,
	// Fehler beim Einstellen der TEDS Anschlussart auf dem MSP
	ERRHAL_MSPSETOWMODE = -120762,
	// Keine Teds Kommunikation moeglich, da (mindestens) ein Connector im Overflow
	ERRHAL_TEDSCOM_NOT_POSSIBLE = -120763,
	// Fehler beim Ermitteln der Seriennumer des ersten Device auf dem 1Wire Bus
	ERRHAL_MSPGETFOWNUM = -120765,
	// Fehler beim Ermitteln der Seriennumer zweiten (drittes...) Device auf dem 1Wire Bus
	ERRHAL_MSPGETNOWNUM = -120770,
	// Fehler beim Lesen von einem 1Wire Baustein
	ERRHAL_MSPGETOWDAT = -120775,
	// Fehler beim Beschreiben eines 1Wire Bausteins
	ERRHAL_MSPSETOWDAT = -120780,
	// Fehler beim MSP Firmware Download
	ERRHAL_MSPFWDWN = -120785,
	// Fehler beim Sprung in den Loader
	ERRHAL_MSPJMPLOAD = -120790,
	// Fehler beim Sprung in die Applikation
	ERRHAL_MSPJMPAPP = -120795,
	// Fehler beim Senden des "Beep" Kommandos
	ERRHAL_MSPBEEP = -120796,
	// Fehler beim Programmieren des Flash (blockweise mit Zuruecklesen)
	ERRHAL_MSPPRGBLKV = -120800,
	// Fehler beim Programmieren des Flash (blockweise ohne Zuruecklesen)
	ERRHAL_MSPPRGBLK = -120805,
	// Fehler beim Lesen des Flash (blockweise)
	ERRHAL_MSPRDBLK = -120810,
	// Fehler beim Vorbereiten eines Lesevorgangs im MSP-Flash
	ERRHAL_MSPPRERDBLK = -120811,
	// Fehler Lesen von RFID Daten
	ERRHAL_MSPRDRFID = -120815,
	// Fehler Schreiben von RFID Daten
	ERRHAL_MSPWRRFID = -120816,
	// Fehler beim Einstellen der LED
	ERRHAL_MSPSETLED = -120817,
	// Fehler Lesen von RFID Daten --> Kein Rfid vorhanden
	ERRHAL_MSPRD_NO_RFID = -120818,
	// Fehler Lesen von RFID Daten --> Rfid vorhanden, aber fehlerhafter Header
	ERRHAL_MSPRD_ERR_RFID = -120819,
	// Kombination aus Adresse und Laenge ueberschreitet die Grenzen des EEEPROMS
	ERRHAL_MSP_EEPROM_SIZE = -120820,
	// Fehler beim Schreiben auf das EEPROM
	ERRHAL_MSPWR_EEPROM = -120821,
	// Fehler beim Lesen vom EEPROM
	ERRHAL_MSPRD_EEPROM = -120822,

	// **********************************************

	// Fehler beim Einstellen des DAU fuer die PLL (SPI-Bus)
	ERRHAL_SPISETDACPLL = -120900,
	// Fehler beim Oeffnen des SPI-Treibers
	ERRHAL_SPIOPENDRV   = -120905,
	// Fehler beim Versuch ein Device auf dem SPI-Bus auszuwaehlen (Slave Select)
	ERRHAL_SPISETSS     = -120910,

	// **********************************************

	ERR_HAL_CALCAL                                      = -130500,
	ERR_HAL_CALMEAS                                     = -130505,
	ERR_HAL_CORRECTPHASE                                = -130510,
	ERR_HAL_CALC_PT_COEFFS                              = -130513,
	ERR_HAL_ACTIVATESET                                 = -130515,
	WARN_HAL_ACTIVATESET                                =  130515,
	WARN_SET_ZERO_INCL_ANALOGOUT                        =  130517,
	WARN_SET_ZERO                                       =  130518,

	WARN_SIG_NOT_AVAILABLE                              =  130519,
	WARN_ADJUST_RMS_PERIOD                              =  130522,

	ERRHAL_LOOP                                         = -130524,


	// Fehler beim Bearbeiten der Temperaturdrift im MX590
	ERRHAL_CORRTEMPERATURE                              = -130526,
	ERRHAL_SET_CJ_OFFSET                                = -130527,   //we. war: -130518,
	ERRHAL_COLDJUNCTION                                 = -130528,   //we. war: -130520,

	ERRHAL_AUTOADJUST                                   = -130530,

	ERRHAL_WRONG_CHANTYPE                               = -130560,
	ERRHAL_NOT_SUPPORTED                                = -130600,


	// Fehler beim Einschalten des Interpolationsfilters (im FPGA)
	ERRHAL_INTERPOLENABLE                               = -130700,
	// Fehler beim Initialisieren des Interpolationsfilters
	ERRHAL_INTERPOLINIT                                 = -130701,
	// Fehler beim Einstellen der Rate fuer das Interpolationsfilter
	ERRHAL_INTERPOLSETDATARATE                          = -130703,
	// Skalierung des Interpolationsfilters fuehrt zum Overflow
	ERRHAL_INTERPOLSETSKALINGOVERFLOW                   = -130704,
	// Fehler beim Skalieren des Interpolationsfilters
	ERRHAL_INTERPOLSETSCALING                           = -130705,
	// Skalierung analoger Ausgang: Nulloffset ausserhalb des einstellbaren Bereichs
	ERRHAL_SCALDACNULL                                  = -130706,
	// Skalierung analoger Ausgang: Gain ausserhalb des einstellbaren Bereichs
	ERRHAL_SCALDACGAIN                                  = -130707,
	// Skalierung analoger Ausgang: Nulloffset ausserhalb des einstellbaren Bereichs
	ERRHAL_SCALDACNULL_MX410                            = -130708,
	// Skalierung analoger Ausgang: Gain ausserhalb des einstellbaren Bereichs
	ERRHAL_SCALDACGAIN_MX410                            = -130709,

	//                                                    -139999
	// **********************************************


	// **********************************************
	// Kernel modules
	// **********************************************

	// dieser Funktionsblock-Typ existiert nicht
	ERR_RT_WRONG_FB_TYPE                                = -140000,
	// zu viele Funktionsblock-Phasen definiert
	ERR_RT_TOO_MUCH_PHASES                              = -140001,
	// falsche Instanznummer
	ERR_RT_WRONG_INSTANCE                               = -140005,
	// zu viele FifoReader-Instanzen
	ERR_RT_TOO_MUCH_FIFOS                               = -140010,
	// zu viele Filterinstanzen
	ERR_RT_TOO_MUCH_FILTERS                             = -140012,
	// zu viele Skalierinstanzen
	ERR_RT_TOO_MUCH_LINSCALINGS                         = -140014,
	// zu viele SampleInterpolationsinstanzen
	ERR_RT_TOO_MUCH_SAMPLEINTERPOLATIONS                = -140016,

	// unbekanntes Kommando
	ERR_RT_CMD                                          = -140020,
	// Fehler beim Setzen von Showbits
	ERR_RT_SET_SHOWBIT	                                = -140021,
	// Fehler beim Kopieren in den User-Mode
	ERR_RT_COPY_TO_USER                                 = -140024,
	// Fehler beim Kopieren aus dem User-Mode
	ERR_RT_COPY_FROM_USER                               = -140025,
	// Falscher Ringpuffertyp fuer CAN-Ringbuffer
	ERR_RT_CAN_RBUF_TYPE                                = -140026,
	// Falscher Index fuer CAN-Ringbuffer
	ERR_RT_CAN_RBUF_INDEX                               = -140027,
	// Pretrigger zu gross fuer Puffergroesse
	ERR_RT_PRETRIGGER                                   = -140030,

	// zu grosser ClientIndex
	ERR_RT_CLIENTINDEX                                  = -140035,
	// allokieren eines isochronen Kanals ist fehlgeschlagen,
	// Kanal ist schon allokiert
	ERR_RT_CHANNEL_ALLOCATE                             = -140200,
	ERR_RT_BANDWIDTH_ALLOCATE		                        = -140201,

	ERR_RT_WRONG_CHANNEL_NUMBER                         = -140202,

	ERR_RT_READ_CHANNEL_REG                             = -140203,

	ERR_RT_CHANNEL_ALREADY_ALLOCATED                    = -140204,

	ERR_RT_BR_WHILE_CHANNEL_ALLOC                       = -140205,
	ERR_RT_CHANNEL_CMP_SWP_FAILED                       = -140206,
	ERR_RT_CHANNEL_LOCK_FAILED                          = -140207,
	// Die Freigabe eines isochronen Kanals ist fehlgeschlagen,
	// Kanal wurde nicht von diesem Device reserviert
	ERR_RT_CHANNEL_RELEASE                              = -140220,
	// Kanal ist schon freigegeben
	ERR_RT_CHAN_ALREADY_AVAILABLE                       = -140300,

	//                                                    -160000

	// **********************************************
	// do not use, because used for texts:              166000 ... 166999
	//                                                  170000 ... 179899
	// **********************************************

	// **********************************************
	// additional connectors
	// **********************************************

	ERR_ADDCONN_MODE_NOT_SUPPORTED                      = -179900,

	// Analog Output
	ERR_ANALOG_OUT_MODE_NOT_SUPPORTED                   = -179910,
	WARN_ANALOG_OUT                                     =  179911,
	WARN_ANALOG_OUT_SCALING_NOT_SUPPORTED_DISABLED      =  179914,
	ERR_ANALOG_OUT_SCALING_NOT_SUPPORTED_DISABLED       = -179915,

	ERR_DIGITALIO_MODE_NOT_SUPPORTED                    = -179920,
	ERR_DIGITALIO_TRIGGER_LEVEL_TOO_LOW                 = -179921,
	ERR_DIGITALIO_TRIGGER_LEVEL_TOO_HIGH                = -179922,

	WARN_SET_DIGITAL_IO_IS_INPUT                        =  179930, //we. war: 130526,
	WARN_SET_DIGITAL_IO_IS_CONNECTED                    =  179931, //we. war: 130527,

	ERR_DIVIDER_MUST_BE_GREATER_THAN_ZERO               = -179940,
	ERR_INTERVAL_MUST_BE_GREATER_THAN_ZERO              = -179941,

	//                                                    -179999

	// **********************************************
	// // do not use, because used for texts:           180000 ... 184999
	// **********************************************

	// **********************************************
	// additional functions
	// **********************************************

	ERR_MATHTYPE_NOT_SUPPORTED                          = -185100,
	WARN_CHECK_ADDFUNC_PARAMETERS                       =  185101,

	WARN_MATH_DISABLED_SIGNALREFERENCE_OFF              =  185102,
	WARN_PEAK_DISABLED_SIGNALREFERENCE_OFF              =  185103,

	ERR_MATH_OUTPUTRATE_NOT_SUPPORTED                   = -185300,
	WARN_MATH_OUTPUTRATE_CHANGED_TO_SIGNALOUTPUTRATE    =  185302,
	WARN_MATH_OUTPUTRATE_CHANGED_TO_MAX                 =  185303,
	WARN_MATH_OUTPUTRATE_CHANGED_TO_ALIGN               =  185304,

	WARN_MATH_RMS_SAMPLECOUNT_CHANGED_TO_MAX            =  185310,
	WARN_MATH_RMS_PERIOD_CHANGED_TO_MIN                 =  185311,

	WARN_MATH_ROTVIBE_OUTPUTFILTER_SWITCHED_OFF         =  185317,
	WARN_MATH_ROTVIBE_OUPUT_HIGHPASS_ALIGNED            =  185318,
	WARN_MATH_ROTVIBE_FILTER_SWITCHED_OFF               =  185319,
	WARN_MATH_ROTVIBE_HIGHPASS_ALIGNED                  =  185320,
	ERR_MATH_INTEGRATION_NOT_SUPPORTED                  = -185321,
	WARN_MATH_SCALINGFACTOR_ZERO                        =  185322,
	ERR_MATH_PULSESPERROUND_LESS_THAN_1                 = -185323,
	ERR_MATH_INTEGRATOR_NOT_ENABLED                     = -185324,
	ERR_MATH_UNIT_NOT_ENABLED                           = -185325,
	WARN_MATH_UNIT_NOT_ENABLED                          =  185325,
	WARN_MATH_MATRIX_IS_USED_THIS_UNIT_DISABLED         =  185326,
	WARN_MATH_USING_MATRIX_WILL_DISABLE_OTHERS          =  185327,
	ERR_MATH_ALARM_HYSTERESIS_NEGATIVE                  = -185328,

	WARN_PEAK_OUTPUTRATE_CHANGED_TO_ALIGN               =  185350,

	ERR_MATH_PARAM_REFUSED                              = -185396,
	ERR_PEAK_PARAM_REFUSED                              = -185397,
	ERR_GATE_PARAM_REFUSED                              = -185398,


	// **********************************************
	// test signal
	// **********************************************

	ERR_TESTSIGNAL_MODE_NOT_SUPPORTED                   = -185400,
	ERR_TESTSIGNAL_AMPLITUDETYPE_NOT_SUPPORTED          = -185402,

	ERR_TESTSIGNAL_AMPLITUDE_LEVEL_MIN_MAX              = -185403,
	ERR_TESTSIGNAL_AMPLITUDE_PEAKPEAK_DELTA             = -185404,

	ERR_TESTSIGNAL_ZERO_DIVISION                        = -185405,
	ERR_TESTSIGNAL_DUTYCYCLE                            = -185406,
	ERR_TESTSIGNAL_FREQUENCY                            = -185407,

	ERR_TESTSIGNAL_WRONG_INDEX                          = -185410,

	WARN_TESTSIGNAL_DUTYCYCLE_SET_TO_MIN                =  185412,
	WARN_TESTSIGNAL_DUTYCYCLE_SET_TO_MAX                =  185413,

	WARN_TESTSIGNAL_PHASE_SET_TO_MIN                    =  185414,
	WARN_TESTSIGNAL_PHASE_SET_TO_MAX                    =  185415,

	ERR_TESTSIGNAL_VOLTAGE_OUT_OF_RANGE                 = -185416,

	ERR_TESTSIGNAL_ARBITRATION_SOURCETYPE_NOT_SUPPORTED = -185420,
	ERR_TESTSIGNAL_ARBITRATION_RUNMODE_NOT_SUPPORTED    = -185421,
	ERR_TESTSIGNAL_ARBITRATION_REPETITIONRATE_NOT_SUPPORTED = -185422,
	ERR_TESTSIGNAL_ARBITRATION_BURSTMODE_NOT_SUPPORTED  = -185423,

	ERR_TESTSIGNAL_PARAM_REFUSED                        = -185499,

	//                                                    -189999

	// **********************************************

//                                            -1999999
	// **********************************************
	// Error codes formerly defined in DSS_FrameDefs.h
	// **********************************************

	ERR_ECAT_FUNCTIONBLOCK_SET_PDO_STATUS     = -1515708,
	ERR_ECAT_FUNCTIONBLOCK_SET_PDO_INDEX      = -1515707,
	ERR_ECAT_FUNCTIONBLOCK_CREATE_PROBLEM     = -1515706,

	ERR_ECAT_RTSIGNALE_NOT_VALID              = -1515705,
	///
	WARN_ECAT_DEMUX_SIGNALE_NOT_VALID         =  1515704,
	///
	WARN_ECAT_MODULE_PARAM_NOT_READY          =  1515703,
	/// Anpassen des Max Iso Limit
	ERR_ECAT_MODULE_STATUS_NOT_READABLE       = -1515702,
	/// Anpassen des Max Iso Limit
	WARN_ECAT_CORRECTED_FW_SIGNALE_LIST_SIZE  =  1515701,
	/// Fehler beim erreichen Max ISO limit
	ERR_ECAT_COUNT_OF_FW_SIGNALS_LIMIT_REACHED = -1515700,

	/// Fehler beim erreichen des NTP Server
	ERR_NTP_SERVER_NOT_REACHED                = -1515600,

	/// Fehler Modulestatus
	ERR_STAT_MODULE                           = -1515500,

	// **********************************************

	// **********************************************
	// // do not use, because reserved/used for texts:    1100000 .. 1290000
	// **********************************************

	/// error while writing to flash memory
	ERR_FW_FLASH_WRITE                    = -1080020,
	/// error while erasing flash memory
	ERR_FW_FLASH_ERASE                    = -1080015,
	/// Firmware file could not be opened
	ERR_FW_OPEN_FILE                      = -1080010,
	/// Msp download (MSP number 0,1-n) failed!
	ERR_FW_NO_MSP_DOWNLOAD                = -1080005,

	// **********************************************

	/// Falsche NEC-Kanal Nummer
	ERR_FW_NO_NEC_CHANNEL                 = -1018005,
	/// NEC Download schlug fehl!
	ERR_FW_NO_NEC_DOWNLOAD                = -1018006,
	/// Falsche MSP-Kanal Nummer
	ERR_FW_NO_MSP_CHANNEL                 = -1018000,
	/// Initjust-Werte im MSP korrupt
	ERR_MSP_DATA_CORRUPT                  = -1017996,
	/// Urjustagewerte wurden vom MSP geladen
	ERR_INITJUST_FROM_MSP                 = -1017995,
	/// Keine gueltigen Urjustagewerte vorhanden (MSP)
	ERR_NO_VALID_INITJUST                 = -1017993,
	/// Urjustagewerte konnten nicht gespeichert werden
	ERR_STORING_INITJUST                  = -1017992,
	/// Urjustagewerte konnten nicht gelesen werden
	ERR_GETTING_INITJUST                  = -1017991,
	/// Vom Msp gelesene Pll Spannung ausserhalb des gueltigen Bereichs
	ERR_PLLVOL_OUTOFRANGE                 = -1017990,
	/// Vom Msp gelesene Pll Spannung korrupt
	ERR_PLLVOL_CORRUPT                    = -1017989,
	/// Pll-Voltage wurde vom MSP geladen
	ERR_PLLVOL_FROM_MSP                   = -1017988,
	/// Daten Adressbustest schlug fehl
	ERR_DATA_ADDR_BUSTEST                 = -1017987,

	// **********************************************

	/// Filename zu lang
	ERR_SYS_FILENAME                      = -1016050,
	ERR_SYS_FILE_TOO_LARGE								= -1016049,
	/// fehlerhafte Uebergabeparameter, z.B. "Null Pointer"
	ERR_SYS_BAD_DELIVERY                  = -1016045,
	/// Logfile konnte nicht geoeffnet werden
	ERR_SYS_OPENLOGFILE                   = -1016040,
	/// Framework wurde nicht gestartet
	ERR_SYS_FRAMEWORK_START               = -1016035,
	/// Watchdog konnte nicht geoeffnet werden
	ERR_SYS_OPEN_CLOSE_WD                 = -1016030,
	/// ungueltige Zeit
	ERR_SYS_INVALID_TIME                  = -1016025,
	/// ungueltiges Handle
	ERR_SYS_INVALID_HANDLE                = -1016020,
	/// Fehler in Systemshell Ausfuehrung
	ERR_SYS_SHELL                         = -1016015,
	/// Fehler bei File I/O
	ERR_SYS_FILEIO                        = -1016010,
	/// Fehler bei Oeffnen einer Pipe
	ERR_SYS_OPENPIPE                      = -1016005,
	/// Fehler bei Schliessen einer Pipe
	ERR_SYS_CLOSEPIPE                     = -1016004,

	// **********************************************

	/// oeffnen des SPI Treibers fehlgeschlagen
	ERR_SPI_OPEN_DRIVER                   = -1015050,
	/// aufwecken des DAC fehlgeschlagen
	ERR_SPI_WAKEUP_DAC                    = -1015049,
	/// schreiben der zu stellenden digits auf den Treiber nicht moeglich
	ERR_SPI_WRITE_DRIVER_DIGITS           = -1015048,
	/// schreiben eines Kommandos auf den Treiber nicht moeglich
	ERR_SPI_WRITE_DRIVER_COMMAND          = -1015047,
	/// Spannung befindet sich ausserhgalb des zulaessigen Bereichs
	ERR_SPI_VOLT_OUTOF_RANGE              = -1015040,
	/// Error setting the PLL-Dac
	ERR_SPI_PLLDAC                        = -1015039,

	/// TimeSource data not valid
	ERR_SYNC_NOT_ALL_MODULES_READY        = -1015031,

	/// TimeSource data not valid
	ERR_SYNC_TIMESOURCE_DATA_NOT_VALID    = -1015030,

	/// Syntax-Error IP address
	ERR_PARAM_IP_ADDRESS_SYNTAX           = -1015023,
	/// Parameter for choice TimeSource not supported
	ERR_PARAM_SYNC_TIMESOURCE_NOT_SUPPORTED = -1015022,

	/// Die Cycle-Counts zwischen Master und Slave sind zu lange zu unterschiedlich
	ERR_SYNC_CYCLEDIFF_TOO_LARGE          = -1015021,
	/// der isochrone Kanal laesst sich nicht reservieren, ein anderes Device hat ihn bereits
	ERR_SYNC_ALLOC_CHANNEL                = -1015020,
	/// isochroner Kanal laesst sich nicht freigeben, ein anderes Device hat den Kanal reserviert evtl. reserviert
	ERR_SYNC_FREE_CHANNEL                 = -1015019,
	/// isochroner Kanal ist bereits freigegeben
	ERR_SYNC_CHAN_AVAILABLE               = -1015018,
	/// der Receive Context fuer den SyncKanal auf einem Slave ist bereits eingerichtet,
	/// kann nicht nochmal geschehen
	ERR_SYNC_RECV_CTX_RUNNING             = -1015017,
	/// Sync Context Fehler beim alloc des Transmit
	ERR_SYNC_ALLOC_TX                     = -1015014,
	/// Sync Context Fehler beim free des Transmit
	ERR_SYNC_FREE_TX                      = -1015013,
	/// Sync Context Fehler beim free des receive
	ERR_SYNC_ALLOC_RX                     = -1015012,
	/// Sync Context Fehler beim free des receive
	ERR_SYNC_FREE_RX                      = -1015011,
	/// Synchronisation laeuft nicht,weil oeffnen des Treibers fehlgeschlagen ist
	ERR_SYNC_DRIVER                       = -1015010,
	/// Device ist nicht Synchron zum Master
	ERR_SYNC_NOT_IN_SYNC                  = -1015009,
	/// Device konnte nicht auf eine Regelspannung von 2,5V initialisiert werden
	ERR_SYNC_INIT_VOLTAGE                 = -1015008,
	/// Die PLL laesst sich nicht nicht mit der Regelspannung ansteuern
	ERR_SYNC_ADJUST_PLL                   = -1015007,
	/// Im Sync Buffer sind noch keine SyncDaten, OffsWrite = OffsBegin
	ERR_SYNC_SYNCBUF_EMPTY                = -1015006,
	/// allgemeiner Sync Context Fehler
	ERR_SYNC_CTX                          = -1015005,
	/// isochroner Xmit Context konnte nicht gestartet werden
	ERR_SYNC_INIT_XMIT_CTX                = -1015004,
	/// keine Sync Daten empfangen in der Sync CB Funktion
	ERR_SYNC_STOP_RECV_CTX                = -1015003,
	///isochroner Xmit Context konnte nicht gestartet werden
	ERR_SYNC_STOP_XMIT_CTX                = -1015002,
	/// keine Sync Daten empfangen in der Sync CB Funktion
	ERR_SYNC_NO_SYNCDAT                   = -1015000,

	// **********************************************

	/// initialisieren der Adj Factoren fuer den Cal Kanal fehlgeschlagen (Nullsignal, Cal Signal)
	ERR_ADJ_SET_INIT_CAL_FACTS            = -1014999,
	/// Einstellen der neuen Korrekturkennlinie fehlgeschlagen
	ERR_ADJ_SET_ACT_JUST_FACTS            = -1014998,
	/// holen der aktuellen Korrekturkennlinie fehlgeschlagen
	ERR_ADJ_GET_ACT_JUST_FACTS            = -1014997,
	/// holen der initialen Kennlinie (Urjustage Kennlinie) fuer den Cal Kanal fehlgeschlagen
	ERR_ADJ_GET_INIT_CAL_FACTS            = -1014996,
	/// holen der initialen Kennlinie (Urjustage Kennlinie) fuer den Mess Kanal fehlgeschlagen
	ERR_ADJ_GET_INIT_MEAS_FACTS           = -1014995,
	/// MeasRange fuer AutoAdjust-Algorithmus nicht gueltig
	ERR_ADJ_MEASRANGE_INVALID             = -1014994,
	/// Messwert fuer AutoAdjust-Algorithmus ausserhalb des Messbereichs
	ERR_ADJ_MEASRANGE_OVERRUN             = -1014993,
	/// AutoAdjust: falsche Connectornummer
	ERR_ADJ_WRONG_CONNECTOR               = -1014992,

	/// ThermoUtils: falsche Connectornummer
	ERR_THERMO_WRONG_CONNECTOR            = -1014982,
	/// Instanzpointer der ThermoUtils ist NULL, also noch nicht initialisiert
	ERR_THERMO_INSTANCE_NULL              = -1014981,

	// **********************************************

	/// ConnectorInfo: falsche Connectornummer
	ERR_CONN_WRONG_CONNECTOR              = -1014801,
	/// ConnectorInfo: pInstance ist NULL
	ERR_CONN_INSTANCE_NULL                = -1014800,

	// **********************************************

	/// Analoger Ausgang in der Begrenzung aufgrund Uebersteuerung des Signaleingangs etc.
	ERR_ANALOGOUT_SIGOVFL                 = -1014602,
	/// Analoger Ausgang im Kurzschluss
	ERR_ANALOGOUT_OCCF                    = -1014601,
	/// Analoger Ausgang in der Begrenzung aufgrund Skalierung etc.
	ERR_ANALOGOUT_OVFL                    = -1014600,

	// **********************************************

	/// InputReference
	WARN_SIGNALREFERENCE_UNKNOWN                      =  1014504,

	/// OnLevel and OffLevel are identical
	ERR_PARAM_LEVELS_IDENTICAL                        = -1014503,

	/// InputReference
	ERR_SIGNALREFERENCE_CONNECTOR_NOT_SUPPORTED       = -1014502,
	WARN_MAX_ISO_SIGNALINDEX_REACHED                  =  1014501,
	ERR_SIGNALREFERENCE_SIGNALINDEX_NOT_SUPPORTED     = -1014500,
	ERR_INDEX_NOT_SUPPORTED                           = -1014499,
	ERR_SIGNALREFERENCE_SIGNALTYPE_NOT_SUPPORTED_OR_SYNTAX_ERROR = -1014498,
	WARN_SIGNALREFERENCE_EMPTY                        =  1014497,
	ERR_MODULEREFERENCE_SYNTAX                        = -1014496,
	ERR_SIGNALREFERENCE_PEAKINDEX_NOT_SUPPORTED       = -1014495,
	ERR_SIGNALREFERENCE_MATHINDEX_NOT_SUPPORTED       = -1014494,
	ERR_SIGNALREFERENCE_ALARMINDEX_NOT_SUPPORTED      = -1014493,
	ERR_MODULEREFERENCE_NOT_LOCAL                     = -1014492,
	ERR_MODULEREFERENCE_NOT_EXTERNAL                  = -1014491,
	ERR_SIGNALREFERENCE_SYNTAX                        = -1014490,

	ERR_PHYSUNIT_NOT_AVAILABLE                        = -1014489,
	ERR_SIGNALREFERENCE_ASYNCSIGNAL_NOT_ALLOWED_OR_SYNTAX_ERROR = -1014488,
	ERR_SIGNALREFERENCE_DIGITALIOINDEX_NOT_SUPPORTED  = -1014488,

	// **********************************************

	ERR_POINTER_NULL                      = -1014405,

	/// Parametrierung: pParamSetting ist NULL, weil noch keine gueltige Parametrierung eingestellt wurde
	ERR_PARAMSETTINGS_NULL                = -1014404,

	/// Parametrierung: Fehler beim Bearbeiten der SyncSignalParams
	ERR_PARAM_CALCSYNCSIGNAL              = -1014403,

	/// Parametrierung: pPlatform ist NULL
	ERR_PARAM_PLATFORM_NULL               = -1014402,

	/// Parametrierung: pAmp ist NULL
	ERR_PAMP_NULL                         = -1014401,

	/// Parametrierung: pAccessHw ist NULL
	ERR_PARAM_ACCESSHW_NULL               = -1014400,

	// Wird gesetzt, wenn Teds-Daten nicht fehlerfrei gelesen werden konnten
	ERR_TEDS_DATA                         = -1014374,
	// Wird gesetzt, wenn kein Teds vorhanden und TedsUsage == TEDS_REQUIRED
	ERR_PARAM_TEDS_NOT_AVAILABLE          = -1014373,
	// Wird gesetzt, wenn kein Teds vorhanden und TedsUsage == TEDS_AUTO
	WARN_PARAM_TEDS_NOT_AVAILABLE         =  1014372,
	// Wird im MX590 gesetzt, wenn die Anforderung kleiner als der max. Druck des Sensors ist
	WARN_PARAM_REQUEST_BELOW_MEASRANGE    =  1014370,

	// **********************************************

	/// warning: control not performed, because peak mode is controlled by internal signal
	WARN_CTRL_PEAKMODE_SIGNALCTRL         =  1014310,
	/// warning: control not performed, because run mode is controlled by internal signal
	WARN_CTRL_RUNMODE_SIGNALCTRL          =  1014309,
	/// Setzen des SignalGenerator nicht moeglich
	ERR_CTRL_UNABLE_TO_SET_SIGNALGENERATOR = -1014308,
	/// Control of Alarm Unit not possible
	ERR_CTRL_UNABLE_TO_CONTROL_ALARM_UNIT = -1014307,
	/// Control of Peak Unit not possible
	ERR_CTRL_UNABLE_TO_CONTROL_PEAK_UNIT  = -1014306,
	/// Nullstellen nicht moeglich
	ERR_CTRL_UNABLE_TO_SET_ZERO           = -1014305,
	/// Allgemeiner Control - Fehler
	ERR_CTRL_UNSPECIFIED                  = -1014302,

	ERR_WRONG_SECURITY_CODE               = -1014301,

	/// Parameterfehler Control
	ERR_CTRL_WRONG_PAR                    = -1014300,

	// **********************************************

	ERR_DECIMALS_GREATER_THAN_SIGNALLENGTH = -1014263,
	ERR_DECIMALS_TOO_GREAT                = -1014262,
	WARN_SIGNALLENGTH_CHANGED             =  1014261,
	WARN_STARTBIT_SIGNALLENGTH_CHANGED    =  1014260,
	ERR_CAN_ECU                           = -1014259,
	ERR_CAN_WRITE_CANSIGNAL               = -1014258,
	ERR_CAN_TRANSMIT_SIGNALREFERENCE_INVALID   = -1014257,
	ERR_CAN_TRANSMIT_MODULEREFERENCE_NOT_LOCAL = -1014256,
	/// Generic error if parameters for CAN transmission are invalid
	ERR_CAN_TRANSMIT_MSG                  = -1014255,
	/// CAN transmission type not supported
	ERR_CAN_TRANSMISSIONTYPE_NOT_254      = -1014254,
	ERR_CAN_TRANSMISSIONTYPE_NOT_SUPPORTED = -1014253,
	/// Connector-Nummer unterstuetzt kein Can
	ERR_CAN_CONNECTORNO                   = -1014252,
	/// Can-Transmit-Channel nicht parametriert oder nicht aktiviert
	ERR_CAN_TRANSMIT_CHANNEL              = -1014250,
	/// User-Can-Nachricht konnte nicht direkt gesendet werden
	ERR_CAN_TRANSMIT_USERDEF_MSG          = -1014245,
	/// Can-Transmit-Tabelle ByteOrder nicht unterstuetzt
	ERR_CAN_TRANSMIT_BYTEORDER_NOT_SUPPORTED = -1014244,
	/// Can-Transmit-Tabelle fehlerhaft
	ERR_CAN_TRANSMIT_TABLE                = -1014243,
	/// Can-Transmitter Wert passt nicht in Byte (groesser als 255)
	ERR_CAN_TRANSMIT_BYTEVALUE            = -1014242,
	/// Can-Transmitter Anzahl der zu sendenden Byte zu gross
	ERR_CAN_TRANSMIT_BYTECOUNT            = -1014241,
	/// Neue CAN-Transmitter-Tabelle konnte nicht per ioctl gesetzt werden
	ERR_CAN_TRANSMITTER_USERDEF_MSG_SET_TABLE = -1014240,
	/// Mode Length > 64
	ERR_CAN_DECODER_MODELENGTH_64         = -1014238,
	/// Mode Startbit > 63
	ERR_CAN_DECODER_MODESTARTBIT_63       = -1014237,
	/// Length > 64
	ERR_CAN_DECODER_LENGTH_64             = -1014236,
	/// Startbit > 63
	ERR_CAN_DECODER_STARTBIT_63           = -1014235,
	/// Der CAN-Identifier ist zu gross
	ERR_CAN_COMBINATION_IDENTIFIER_FRAMEFORMAT = -1014234,
	/// CAN-Kanal-Nummer nicht vorhanden
	ERR_CAN_CHANNELNO                     = -1014233,
	/// Validierung der CAN-Dekoder-Tabelle ergab, dass die Summe aus Mode-Startbit und Mode-Length ungueltig ist
	ERR_CAN_DECODER_MODE_STARTBIT_LENGTH  = -1014232,
	/// Validierung der CAN-Dekoder-Tabelle ergab, dass die Summe aus Startbit und Length
	///  ungueltig ist
	ERR_CAN_COMBINATION_STARTBIT_LENGTH = -1014231,
	/// Validierung der CAN-Dekoder-Tabelle ergab, dass die Laenge der auszuwertenden
	///  Rohdaten nicht gleich 64 ist, obwohl die Rohdaten als double behandelt werden sollen
	ERR_CAN_DECODER_FORMAT_DOUBLE         = -1014230,
	/// Validierung der CAN-Dekoder-Tabelle ergab, dass die Laenge der auszuwertenden
	///  Rohdaten nicht gleich 32 ist, obwohl die Rohdaten als float behandelt werden sollen
	ERR_CAN_DECODER_FORMAT_FLOAT          = -1014229,
	/// Validierung der CAN-Dekoder-Tabelle ergab, dass diese Bedingung nicht erfuellt:
	/// Wenn die Laenge der auszuwertenden Rohdaten groesser als 32 ist,
	///  muessen die Datentypen fuer die Rohdaten und die Signalwerte gleich sein.
	ERR_CAN_DECODER_FORMAT_32             = -1014228,
	/// Validierung der CAN-Dekoder-Tabelle ergab, dass der Index auf einen
	///  Signal-Ringpuffer ausserhalb der Gueltigkeit liegt
	ERR_CAN_DECODER_RBUFNO_INVALID        = -1014227,
	/// Validierung der CAN-Dekoder-Tabelle ergab, dass ein Signal-Ringpuffer
	///  doppelt verwendet werden soll
	ERR_CAN_DECODER_RBUFNO_USED_TWICE     = -1014226,
	/// Aktuelle CAN-Dekoder-Tabelle konnte nicht per ioctl geholt werden
	ERR_CAN_DECODER_GET_TABLE             = -1014225,
	/// Neue CAN-Dekoder-Tabelle konnte nicht per ioctl gesetzt werden
	ERR_CAN_DECODER_SET_TABLE             = -1014224,
	/// CAN-Dekoder-Tabelle ist voll
	ERR_CAN_DECODER_TABLE_FULL            = -1014223,
	/// CAN-Dekoder konnte nicht deaktiviert werden
	ERR_CAN_DECODER_DISABLE               = -1014222,
	/// CAN-Dekoder konnte nicht aktiviert werden
	ERR_CAN_DECODER_ENABLE                = -1014221,
	/// CAN-Dekoder-Tabelle mit Dekodiervorschriften ungueltig
	ERR_CAN_DECODER_TABLE_NOT_VALID       = -1014220,
	/// Fehler beim Parametrieren eines CAN-Signals
	ERR_CAN_DECODER_PARAM_CAN_SIGNAL      = -1014219,
	/// CAN-Controller reagiert nicht mehr
	ERR_CAN_NO_RESPONSE                   = -1014218,
	/// CAN Controller fordert Reset an
	ERR_CAN_RESET_REQUEST                 = -1014217,
	/// CAN-OVERRUN transmit controller or transmitter ringbuffer
	ERR_CAN_TRANSMITTER_OVERRUN           = -1014216,
	/// CAN-BUS-ERROR im Transmitter
	ERR_CAN_BUS_ERROR_TRANSMITTER         = -1014215,
	/// CAN-BUS-WARNING im Transmitter
	ERR_CAN_BUS_WARNING_TRANSMITTER       = -1014214,
	/// CAN-OVERRUN receive controller or receive ringbuffer
	ERR_CAN_RECEIVER_OVERRUN              = -1014213,
	/// CAN-BUS-OFF
	ERR_CAN_BUS_OFF                       = -1014212,
	/// CAN-BUS-ERROR im Receiver
	ERR_CAN_BUS_ERROR                     = -1014211,
	/// CAN-BUS-WARNING im Receiver
	ERR_CAN_BUS_WARNING                   = -1014210,

	/// Fehler beim Initialisieren der Parameter fuer die CAN-Signal-Puffer
	ERR_CAN_INIT_CANSIGNAL_PARAMS         = -1014209,
	/// CAN-Bitrate konnte nicht aus Treiber geholt werden
	ERR_CAN_GET_BITRATE                   = -1014208,
	/// CAN-Status konnte nicht aus Treiber geholt werden
	ERR_CAN_GET_STATE                     = -1014207,
	/// CAN-Treiber hat ungueltiges Handle/wurde nicht geoeffnet
	ERR_CAN_INVALID_HANDLE                = -1014206,
	/// Index zu CAN-Device existiert nicht
	ERR_CAN_INDEX                         = -1014205,
	/// CAN-Treiber allgemeiner Fehler bei IOCTL
	ERR_CAN_IOCTL                         = -1014204,
	/// CAN-Bitrate wird nicht unterstuetzt
	ERR_CAN_BITRATE_NOT_AVAILABLE         = -1014203,
	/// CAN-Controller konnte nicht deaktiviert werden
	ERR_CAN_DISABLE                       = -1014202,
	/// CAN-Controller konnte nicht aktiviert werden
	ERR_CAN_ENABLE                        = -1014201,
	/// oeffnen des CAN-Treibers fehlgeschlagen
	ERR_CAN_OPENDRV                       = -1014200,

	// **********************************************

	/// Fehler beim Lesen/Schreiben der Fbfs-Daten fuer LED-Parameter
	ERR_FBFS_ACCESS_LED                   = -1014191,
	/// Fehler beim Lesen/Schreiben der Fbfs-Daten fuer CAN-Parameter
	ERR_FBFS_ACCESS_CAN                   = -1014190,

	// **********************************************

	/// Fehler beim Setzen des FreezedFlag
	ERR_DAQ_SET_FREEZED_FLAG              = -1014142,
	/// Fehler beim Setzen des RtMeasMode
	ERR_DAQ_SET_RTMEASMODE                = -1014141,
	/// der gewuenschte Start der Messung liegt zu weit in der Vergangenheit
	ERR_DAQ_TRIGTIME                      = -1014140,
	ERR_DAQ_TRIGGERUNKNOWN                = -1014139,
	/// Version mismatch of IsoSignalList
	ERR_DAQ_ISOSIGNALLIST_VERSION         = -1014138,
	/// Pointer auf pAmp ist NULL
	ERR_DAQ_AMP_NULL                      = -1014137,
	/// Pointer auf m_pInstance von iDataAcqResprSrvCb ist NULL, also noch nicht initialisiert
	ERR_DAQ_DATAACQRESPRSRV_NULL          = -1014135,
	/// Pointer auf m_pDriver ist NULL, also noch nicht initialisiert
	ERR_DAQ_RTDRIVER_NULL                 = -1014133,
	/// Kanal-Typ nicht definiert
	ERR_DAQ_WRONG_CHAN_TYPE               = -1014131,

	/// Fehler beim Lesen der Demodulations-Info
	ERR_DAQ_GET_DEMOD_INFO                = -1014127,
	/// Fehler beim Setzen des Synchronisations-Puffers
	ERR_DAQ_SET_SYNCBUF                   = -1014126,
	/// Fehler beim Setzen des Error-Puffers
	ERR_DAQ_SET_ERRORBUF                  = -1014125,

	/// Fehler beim Setzen der Pt-Element-Koeffizienten in den Echtzeittreiber
	ERR_DAQ_DRV_SET_PT_COEFFS             = -1014124,
	/// Fehler beim Setzen der Thermoelement-ColdJunction in den Echtzeittreiber
	ERR_DAQ_DRV_SET_COLDJUNCTION          = -1014123,
	/// Fehler beim Setzen der ActJust-Koeffizienten in den Echtzeittreiber
	ERR_DAQ_DRV_SET_INITJUST              = -1014122,
	/// Fehler beim Lesen eines elektrischen Rohwertes aus dem Echtzeittreiber
	ERR_DAQ_GET_ELECTRICALVALUE           = -1014121,
	/// zu viele Koefizienten fuer ein Skalier-Polynom
	ERR_DAQ_SCALPOLY_COEFFS               = -1014120,
	/// zu viele Segmente fuer ein Skalier-Polynom
	ERR_DAQ_SCALPOLY_SEGMENTS             = -1014118,
	/// zu viele Segmente fuer eine Linear-Tabellen-Skalierung
	ERR_DAQ_SCAL_LINTABLE_SEGMENTS        = -1014117,
	/// falsche Version der Echtzeittreibers
	ERR_DAQ_WRONG_RTDRIVER                = -1014115,
	// error setting internal scaling
	ERR_DAQ_SCALING_INTERNAL              = -1014113,
	/// Skalierungsart auf diesem Signaltyp nicht moeglich
	ERR_DAQ_SCALETYPE                     = -1014112,
	/// falsche Kanalnummer
	ERR_WRONG_CHANNEL_INDEX               = -1014111,
	/// falscher Signal-Index
	ERR_WRONG_SIG_INDEX                   = -1014110,
	/// falsche Connectornummer
	ERR_WRONG_CONNECTOR                   = -1014109,

	/// Urjustageparameter nicht gefunden
	ERR_DAQ_INITJUST_NOTFOUND             = -1014108,
	/// Nummer des Kalibrierpunkts passt nicht zu aktueller Verstaerkereinstellung
	ERR_DAQ_INITJUST_POINTNR              = -1014107,
	/// Urjustageparameter ungueltig
	ERR_DAQ_INITJUST_WRONGPAR             = -1014106,
	/// Signal-ID zu lang
	ERR_DAQ_SIGID_LEN                     = -1014105,
	/// Signal-ID unbekannt
	ERR_DAQ_SIGID_UNKNOWN                 = -1014102,
	/// Fehler beim Parametrieren eines Filters aufgetreten
	ERR_DAQ_FILTER                        = -1014100,
	/// Signal nicht verfuegbar
	ERR_DAQ_SIGNAL_NOT_AVAILABLE          = -1014099,
	/// Fehler beim Setzen eines FuncBlock-Typs
	ERR_DAQ_ADD_FBTYPE                    = -1014098,
	/// Fehler beim Erzeugen einer Funcblock-Instanz
	ERR_DAQ_ADD_FB_INSTANCE               = -1014097,
	/// Fehler beim Setzen des FuncBlock-Inputs
	ERR_DAQ_SET_FB_INPUT                  = -1014096,
	/// gewaehlte Ausgaberate fuer das Signal nicht moeglich
	ERR_DAQ_OUTRATE                       = -1014095,
	/// Fehler beim Setzen der Ringpuffer-Parameter
	ERR_DAQ_SET_BUFFER_PARS               = -1014094,
	/// Fehler beim Stellen des Nullwerts
	ERR_DAQ_SET_ZEROVAL                   = -1014093,
	/// Fehler beim Setzen der Fiforeader-Parameter
	ERR_DAQ_FIFO_PARAMS                   = -1014092,
	/// Fehler beim Setzen der Fifireader-Cal-Parameter
	ERR_DAQ_FIFOREADER_CAL_PARAMS         = -1014091,
	/// Fehler beim Setzen der Skalier-Parameter
	ERR_DAQ_SCALING_PARAMS                = -1014090,
	/// Fehler beim Setzen der Module-ID fuer isochronen Transfer
	ERR_DAQ_SET_ISO_MODULE_ID             = -1014089,
	/// Fehler beim Setzen des isochronen Transfers
	ERR_DAQ_SET_ISO_TRANSMIT              = -1014088,
	/// Fehler beim Lesen des isochronen Transfers
	ERR_DAQ_GET_ISO_TRANSMIT              = -1014087,
	/// Fehler beim Initialisieren des Ringpuffers
	ERR_DAQ_INIT_RINGBUF                  = -1014086,

	/// Buffer overflow occured. Data was not read from the buffer in time.
	ERR_DAQ_OVERRUN                       = -1014085,

	ERR_DAQ_UNDEFINED                     = -1014082,
	/// Sync-Fehler FIFO
	ERR_DAQ_FIFO_SYNC                     = -1014070,
	/// Uebersteuerung
	ERR_DAQ_OVERLOAD                      = -1014060,
	/// Speisung in der Begrenzung
	ERR_DAQ_PROCESS                       = -1014061,
	/// Speisung in der Begrenzung
	ERR_DAQ_OCCF                          = -1014055,
	/// Aufnehmerversorgung in der Begrenzung
	ERR_DAQ_LIMSE                         = -1014050,

	ERR_DAQ_NOSIGNALSUBSCRIBED            = -1014030,

	/// die dem Service uebergebene Puffergroesse fuer die Messdaten ist zu klein
	ERR_DAQ_BUFFERSIZE                    = -1014025,

	/// Verstaerkerkanal zu diesem Quellsignal ist ausgeschaltet
	ERR_DAQ_AMPOFF                        = -1014023,

	/// oeffnen des Treibers fuer den Echtzeitteil fehlgeschlagen
	ERR_DAQ_OPENRTDRV                     = -1014010,

	/// dieser Client hat bereits eine Messung laufen
	ERR_DAQ_SUBSCR_RUNNING                = -1014006,

	/// zu dieser UUID ist keine Subscription angelegt
	ERR_DAQ_SUBSCR_UUID                   = -1014005,

	/// Subscription ist im falschen Zustand:
	/// es muss immer die Abfolge openSubscription(), addSignal() und
	/// finishSubscription() eingehalten werden
	ERR_DAQ_SUBSCR_STATE                  = -1014004,

	/// Subscription nicht moeglich (es laufen bereits zu viele Messungen)
	ERR_DAQ_SUBSCRITION                   = -1014002,

	ERR_DATA_FORMAT_NOT_SUPPORTED         = -1010000,

	// **********************************************

	/// Fehler Services Status
	/// es gab ein Problem mit dem Composer
	ERR_SRV_STAT_COMPOSER                 = -1004305,

	// **********************************************

	/// keine Restartfunktion eingehaengt
	ERR_MTHD_SYS_NORESTARTFKT             = -1004260,
	/// falsche Kanalnummer
	ERR_MTHD_SYS_WRONGCHANTYPE            = -1004242,
	/// falsche Kanalnummer
	ERR_MTHD_SYS_WRONGCHAN                = -1004240,
	/// Es laeuft bereits ein Flashing Thread
	ERR_MTHD_SYS_STILLFLASHING            = -1004210,

	ERR_MTHD_SYS_INVALIDFWFILE            = -1004208,
	/// FW Datei zu gross fuer die MTD Partition
	ERR_MTHD_SYS_FWFILETOOLARGE           = -1004207,
	/// File not found
	ERR_MTHD_SYS_FILENOTFOUND             = -1004201,
	ERR_OPENFILE                          = -1004200,
	/// Treiber ist nicht geladen
	ERR_MTHD_SYS_NO_DRIVER                = -1004195,

	/// Unexpexted parameter type
	ERR_PAR_WRONG_TYPE                    = -1004153,
	/// No default settings available for this view
	WARN_PAR_NODEFSET                     = 1004148,
	/// ID fur ParameterView existiert bereits
	ERR_PAR_VIEWID_INUSE                  = -1004146,
	/// Fehler beim Lesen des Parameterwertes
	ERR_PAR_READING_VAL                   = -1004144,
	/// View ist nicht bekannt
	ERR_PAR_VIEWINVALID                   = -1004142,
	/// Contexthandler im falschen Xpath-State
	ERR_PAR_WRONGXPATHSTATE               = -1004140,
	/// Fehler in einem Xpath
	ERR_PAR_XPATHINVALID                  = -1004138,
	ERR_PAR_NOROOT                        = -1004136,
	/// Beim Parsen der XML-Parameter wurde eine Warning erzeugt
	WARN_PAR_PARSING                      =  1004134,
	/// Beim Parsen der XML-Parameter wurde ein Error erzeugt
	ERR_PAR_PARSING                       = -1004135,
	/// Falsche Adresse fuer Multiplexer
	ERR_PAR_INVALIDADDR                   = -1004118,
	/// In diesem Kontext unbekannter Tagname
	WARN_PAR_TAGUNKOWN                    =  1004112,
	/// In diesem Kontext unbekannter Tagname
	ERR_PAR_TAGUNKOWN                     = -1004111,
	/// Parameterwert ausserhalb des gueltigen Bereiches
	ERR_PAR_OUTOFRANGE                    = -1004110,
	/// Falsches Attribut
	ERR_PAR_INVALIDATTR                   = -1004109,
	WARN_PAR_READONLY                     =  1004108,
	/// Falscher Wert fuer ChosenParameter
	ERR_PAR_CHOICEINVALID                 = -1004107,
	/// Fehler in XML-Parametrierung, Kontext verloren
	ERR_PAR_CNTXTLOST                     = -1004106,
	/// Parametrierung enthaelt Fehler und wurde abgewiesen, urspruengliche Parameter
	/// konnten nicht restauriert werden
	ERR_PAR_RESTOREFAILED                 = -1004104,
	/// Parametrierung enthaelt Fehler und wurde abgewiesen
	ERR_PAR_XMLINVALID                    = -1004103,

	/// Es sind momentan keine Messdaten verfuegbar
	WARN_MTHD_DAQ_NODATA                  =  1004010,

	// **********************************************

	/// TxContainer zu klein fuer die uebergebene Datenmenge
	ERR_TXTELCONTAINER_TOO_SMALL          = -1003001,

	// **********************************************

	/// eXpat Parser konnte nicht interpretieren
	ERR_PARSE_PROCESSING_DATA             = -1002002,
	/// eXpat Parser hat ungueltige Daten
	ERR_PARSE_INVALID_DATA                = -1002001,

	// **********************************************

	/// File konnte nicht aus Flash restauriert werden
	ERR_UTIL_FILE_RESTORE                 = -1001140,
	/// Ungueltiges Handle fuer FileItem
	ERR_UTIL_INVALID_FILEHANDLE           = -1001136,
	/// Fehler bei String-Konvertierung
	ERR_UTIL_STRCONVERT                   = -1001132,
	/// String passt nicht zum Pattern
	ERR_UTIL_NO_PATTERN_MATCH             = -1001131,
	/// wait auf EvenFlag lieferte Timeout
	ERR_EVFLG_TIMEOUT                     = -1001130,
	/// Zugriff auf Timers mit falschem Handler
	ERR_UTIL_TIMER_INVALID                = -1001125,
	/// timer was running when being unregistered
	WARN_UTIL_TIMER_WASRUNNING            =  1001025,
	/// die gewuenschte Sprache wird von dem Modul nicht unterstuetzt
	ERR_UTIL_LANGUAGE                     = -1001110,

	// **********************************************
	/// XML-Composer: TAG ist nicht definiert
	ERR_COMPOSER_TAG_NOT_DEFINED          = -1001002,

	// **********************************************

	/// Receive auf einer NodeID, fuer die es noch keinen TelHandler gibt
	ERR_1394_REC_UNKOWNNODEID               = -1000745,
	/// doppelte UUID auf verschiedenen Hardwaren
	ERR_1394_SAMEUUID_DIFFHW                = -1000740,
	/// Device an das gesendet werden sollte wurde waehrenddessen abgezogen
	ERR_1394_SND_DEVDISCONNECTED            = -1000735,
	/// Timeout beim warten auf Deviceupdate bei Sendefehler
	ERR_1394_WAIT4DEVUPDT_TIMEDOUT          = -1000730,
	/// Treiber konnte nicht geladen werden
	ERR_1394_OPENDRIV                       = -1000725,
	/// 1394TelHandler konnte nicht upgedated werden
	ERR_1394_UPDATETELHNDLR                 = -1000720,
	/// 1394TelHandler konnte nicht ausgetragen werden
	ERR_1394_REMOVETELHNDLR                 = -1000715,
	/// 1394TelHandler konnte nicht angelegt werden
	ERR_1394_ADDINGTELHNDLR                 = -1000710,
	/// Fehler: Folgesegment ohne Startsegment empfangen
	ERR_1394_FOLLOWWITHOUTSTARTSEG          = -1000705,
	/// Fehler: alte Telegramfragmente bei neuem Telegramstart uebrig
	ERR_1394_OLDRXTEL_ONSTARTSEG            = -1000700,

	/// Warning: Invalid device generation count
	WARN_1394_THESYCON_INVALID_GENERATION   =  1000699,
	/// Warning: Lost handle to explizit device
	WARN_1394_THESYCON_LOST_HANLDE          =  1000698,
	/// Error: Notification completion event not responsed
	ERR_1394_THESYCON_NOTIFIER_NO_RESPONSE  = -1000696,
	/// Error: Notification receiving data event trigger failed
	ERR_1394_THESYCON_NOTIFIER_NOT_TRIGGERED = -1000694,
	/// Error: Notification Events for receive data is not initialized
	ERR_1394_THESYCON_NOTIFIER_NO_EVENTS    = -1000693,
	/// Error: Notifier initialization failed
	ERR_1394_THESYCON_NOTIFIER_ERROR        = -1000692,
	/// ERROR: Initialization of explizit param failed
	ERR_1394_THESYCON_INIT_FAILED           = -1000691,
	/// Initial Error Codes from Thesycon Adapter
	ERR_1394_THESYCON                       = -1000690,

	// **********************************************

	/// kein netzwerk adapter
	ERR_NET_NETWORKADAPTER                  = -1000680,

	// **********************************************

	/// Fehler bind
	ERR_UDP_BINDTODEVICE                    = -1000658,
	/// Fehler bind
	ERR_UDP_BIND                            = -1000656,
	/// Fehler setsockopt broadcast
	ERR_UDP_SETBROADCAST                    = -1000654,
	/// Fehler setsockopt reuseadr
	ERR_UDP_SETREUSEADR                     = -1000652,
	/// Fehler bei socket()
	ERR_UDP_OPENSOCK                        = -1000650,

	// **********************************************

	/// Fehler bind
	ERR_TCP_BIND                            = -1000644,
	/// Fehler listen
	ERR_TCP_LISTEN                          = -1000642,
	/// Fehler close
	ERR_TCP_CLOSESOCK                       = -1000640,
	/// Fehler checkIpAdr
	ERR_TCP_CHECKIP                         = -1000638,
	/// Fehler getadr
	ERR_TCP_GETADR                          = -1000636,
	/// Fehler getPort
	ERR_TCP_GETPORT                         = -1000635,
	/// Fehler setsockopt reuseadr
	ERR_TCP_SETREUSEADR                     = -1000634,
	/// Fehler setsockopt NODELAY
	ERR_TCP_SETNODELAY                      = -1000632,
	/// Fehler setsockopt nonblockin
	ERR_TCP_SETNONBLOCK                     = -1000630,
	/// Fehler bei socket()
	ERR_TCP_OPENSOCK                        = -1000628,
	/// Fehler bei Connect auf TCP-Socket
	ERR_TCP_CONNECT                         = -1000626,
	/// Fehler bei Accept auf TCP-Socket
	ERR_TCP_ACCEPT                          = -1000624,
	/// Fehler: connect an einen TCP-Teilnehmer misslungen
	ERR_TCP_CONNECT_FAILED                  = -1000620,
	/// Fehler: TCP-Adresse unbekannt
	ERR_TCP_ADR_UNKNOWN                     = -1000612,
	/// Fehler beim Versenden von Telegrammhandler
	ERR_TCP_TELHNDLR_SEND                   = -1000610,
	/// Tcp-Socket liefert auf ein receive keine Daten zurueck
	ERR_TCP_SOCKET_NODATA                   = -1000605,

	ERR_TCP_SOCKET_EINTR                    = -1000604,
	/// Socket verbunden
	ERR_TCP_SOCKET_DESCRIPTOR               = -1000602,

	// **********************************************

	/// Modul nicht in Connectionliste gefunden
	ERR_CON_MODULENOTFOUND                = -1000528,
	/// Fehler beim Empfang von Schnittstelle
	ERR_CON_REC                           = -1000524,
	/// Fehler beim Senden auf Schnittstelle
	ERR_CON_SENDING                       = -1000522,
	/// Message unbekannt
	ERR_CON_UNKNOWNMSG                    = -1000520,
	///  ConnectionHandler: invalid Telegram type
	ERR_CON_INVALID_TELTYPE               = -1000510,
	/// kein gueltiger Pointer auf Telegrammhandler uebergeben
	ERR_CON_TXTELCONTAINER                = -1000506,
	/// Initiale Adress-Resolution ohne Erfolg
	ERR_CON_ADRRESINI_FAILED              = -1000504,
	/// ungueltige Connection uebergeben
	ERR_CON_CONHNDLR_CONINVALID           = -1000502,
	/// unbekannte UUID uebergeben
	ERR_CON_CONHNDLR_UUIDINVALID          = -1000500,

	// **********************************************

	/// ExtAdrResol falsche Telegramlaenge
	ERR_ADR_EXTRESOL_TELSIZE              = -1000474,
	/// Fehler beim Auslesen des AdrRes-Headers
	ERR_ADR_READING_TELHEADER             = -1000470,
	/// AdrRequestor fuer RESPONSE oder Timeout nicht gefunden
	ERR_ADR_NOREQUESTOR                   = -1000468,

	// Fehler Transactions
	/// Fehler beim Auslesen eines Headers aus einem RxTelContainer
	ERR_TRANS_READINGHDR                  = -1000430,
	/// Ein in dem State unerwartetes Event ist aufgetreten
	ERR_TRANS_WRONGEV_FORSTATE            = -1000428,
	/// Transaktion nicht gefunden
	ERR_TRANS_NOTFOUND                    = -1000420,
	ERR_TRANS_CANCELED                    = -1000419,
	/// Timeout fuer Requestor-Transction
	ERR_TRANS_REQ_TIMEOUT                 = -1000418,
	/// Adressresolution fehlgeschlagen
	ERR_TRANS_ADDR_RES                    = -1000416,
	/// falsches Telegramformat
	ERR_TRANS_INVALID_TELFORMAT           = -1000414,
	/// Transaction request received invalid protocol
	ERR_TRANS_REQUESTOR_WRONGPROTTYPE     = -1000404,

	// **********************************************

	WARN_SRV_LISTEN_TO_SOCKET_ALREADY_CONNECTED         =  1000320,
	WARN_SRV_LISTEN_TO_SOCKET_CREATE_TELHANDLER_FAILED  =  1000319,
	WARN_SRV_CONNECT_TO_TCP_CREATE_TELHANDLER_FAILED    =  1000318,
	WARN_SRV_CONNECT_TO_TCP_CREATE_SOCKET_FAILED        =  1000317,
	WARN_SRV_ACCEPT_CLIENT_RETURNED_WITH_ERROR          =  1000316,

	// **********************************************

	/// Keine JustFacts Checksumme im MSP gespeichert, noch nicht aufgerufen ueber Control beim Einmessen
	ERR_NO_MSP_CHKSUM                     = -1000301,
	/// Control unbekannt
	ERR_SRV_CONTROL                       = -1000300,
	/// keine Daten verfuegbar
	ERR_SRV_NODATA                        = -1000275,

	ERR_SRV_NORESPONSE_YET                = -1000271,
	/// kein Request-Telegram zu versenden
	ERR_SRV_REQRMTHD_NOREQTEL             = -1000265,
	/// kein Speicher mehr
	ERR_SRV_MEMORY                        = -1000260,
	/// uebergebene Parameter fehlerhaft
	ERR_SRV_PARAMERROR                    = -1000255,
	/// uebergebene Parameter fehlerhaft
	WARN_SRV_PARAMERROR                   =  1000255,
	/// Service existiert schon
	ERR_SRV_INI_SRVEXISTS                 = -1000223,
	/// falsche Workernummer: Operation iFrame::assignWorker()
	/// die Workernummer wird ab 0 gezaehlt und die Worker mussen vorher mit
	/// createWorker erzeugt worden sein
	ERR_SRV_INI_WRONGWRKNUM               = -1000220,
	/// unbekanntes Kommando in der Antwort
	ERR_SRV_MTHDOUT_UNKNOWNMTHD           = -1000203,
	/// empfangene Antwort passt nicht zum abgeschickten
	/// Kommando: Srv oder MTHD stimmen nicht
	ERR_SRV_MTHDOUT_WRONGANSW             = -1000202,
	/// bereitgestellter Datenpuffer fuer Empfangsdaten (bei ausgehendem Kommando)
	/// zu klein
	ERR_SRV_MTHDOUT_TOOMUCH               = -1000201,

	/// Fehler Assign Worker: Service nicht gefunden
	ERR_SRV_SRV_NOT_FOUND                 = -1000110,
	/// Fehler clCmdWorker: unbekannter Service
	ERR_SRV_UNKNOWNSRV                    = -1000107,
	/// Fehler clMTHDWorker: unbekanntes Command
	ERR_SRV_UNKNOWNMTHD                   = -1000106,
	/// ungueltiger Index
	ERR_INDEX_OUT_OF_RANGE                = -1000000,

};

struct data_header {
    uint8_t     format;
    uint8_t     data_type;
    uint16_t    stream_id;
    uint32_t    sample_count;
    uint64_t    ts[2];
} __attribute__((packed));

enum {
    DATA_FORMAT_SYNC = 1,
    DATA_FORMAT_ASYNC = 2,
};

enum {
    DATA_TYPE_FLOAT = 0x14,
    DATA_TYPE_DOUBL = 0x18,
    DATA_TYPE_INT8 = 0x21,
    DATA_TYPE_INT16 = 0x22,
    DATA_TYPE_INT32 = 0x24,
    DATA_TYPE_INT64 = 0x28,
    DATA_TYPE_CANRAW = 0x3E,
    DATA_TYPE_FLOAT_ASYNC = 0x5C,
    DATA_TYPE_DOUBLE_ASYNC = 0x50,
    DATA_TYPE_INT32_ASYNC = 0x6C,
    DATA_TYPE_INT64_ASYNC = 0x60,
    DATA_TYPE_UINT32_ASYNC = 0xEC,
    DATA_TYPE_UINT64_ASYNC = 0xE0,
    DATA_TYPE_NAN = 0x00,
};

enum {
    DEBUG_LOG_LEVEL_DEBUG2,
    DEBUG_LOG_LEVEL_DEBUG1,
    DEBUG_LOG_LEVEL_DEBUG,
    DEBUG_LOG_LEVEL_INFO,
    DEBUG_LOG_LEVEL_WARNING,
    DEBUG_LOG_LEVEL_ERROR,
    DEBUG_LOG_LEVEL_ALWAYS
};

int recv(int, void *, int, int);
int send(int, void *, int, int);

]]
