#!/usr/bin/lua

--! The network daemon.
--!
--! This daemon handles network stuff.  It is started from master.
--!
--! \file

local ev = require 'ev'
local dbus = require 'dbus'
local sqlite3 = require 'lsqlite3'
local unix = require 'unix'
local struct = require 'struct'
local lpeg = require 'lpeg'
local loop = ev.Loop.default
local net = require 'net_noffi'
local cjson = require 'cjson.safe'
local bit = require 'bit'
local uuid = require 'uuid'
local zlib = require 'zlib'

local _deflate = zlib.deflate
local _inflate = zlib.inflate

local function deflate(data)
    local buffer = ''
    local dstream = _deflate(function(d) buffer = buffer .. d end)
    dstream:write(data)
    dstream:close()
    return buffer
end

local function inflate(data)
    local istream = _inflate(data)
    local ret
    while true do
        local b = istream:read()
        if not b or #b == 0 then break end
        ret = (ret or '') .. b .. '\n'
    end
    istream:close()
    return ret
end

local __ = require '__'
local log = require 'log'

local ffi = require 'ffi'
local C = ffi.C
ffi.cdef[[
int sleep(int);
]]

local NetworkDB = require 'network.db'

local config = require 'system_config'

local ext_if = config.network.ext_if
local int_if = config.network.int_if

ev.Signal.new(function() end, unix.SIGPIPE):start(loop) -- ignore SIGPIPE
ev.Signal.new(function() loop:unloop() end, unix.SIGINT):start(loop)
ev.Signal.new(function() loop:unloop() end, unix.SIGTERM):start(loop)

local bus = dbus.bus_get('system')
bus:hook_libev_default_loop()

local busobj = assert(bus:auto_proxy('org.freedesktop.DBus',
                                     '/org/freedesktop/DBus'))

log:log(log.INFO, "Network daemon starting.")

-- Wait for and read config info from network_config process
local network_config
repeat
    C.sleep(1)
    local jsonf = io.open('/var/run/network_config.json', 'r')
    if jsonf then
        local json = jsonf:read('*a')
        if json then
            network_config = cjson.decode(json)
        end
        jsonf:close()
    end
until network_config
local int_index = network_config.internal_if_index

--! A D-Bus-accessible class for Armada network services.
--!
--! This class is exported over D-Bus from bus name
--! <tt>com.somat.armada.Network</tt> and object path
--! <tt>/com/somat/aramda/Network</tt>.
--? class com.somat.armada.Network
local Network = { }

--? alias Network com.somat.armada.Network
Network.__index = Network
local Network_iface = dbus.Interface.new('com.somat.armada.Network')

Network.normalize_address = net.normalize_ipv6

function Network.new(db) --? private
    local self = setmetatable({ }, Network)
    self.hosts = {}
    self.hosts.localhost = {localhost = true}

    self.int_ip = net.normalize_ipv6(network_config.internal_ip)
    self.hosts[self.int_ip] = self.hosts.localhost

    self.db = NetworkDB:new(self, db, network_config.internal_hostname,
                            self.int_ip)

    -- Set up a timer to retrieve network info and add the record once an IPv4
    -- address has been received
    ev.Timer.new(function(l, t, e)
            if not self.db.platform_info or
                not self.db.platform_info.ip_address then
                self.db:do_get_platform_info()
            end
            if self.db and self.db.platform_info and
                self.db.platform_info.ip_address then
                t:stop(l)
                local rrs = self:QueryRR(self.db.hostname, 'TXT',
                    'platform_info*', '*')
                -- Remove any that already exist.  NOTE:  This only happens if
                -- the network daemon is stopped and restarted.
                for _, rr in ipairs(rrs) do
                    self:RemoveRR(rr[1], rr[2], rr[3])
                end
                -- Add the new one
                self:ModifyRRs({},
                    {
                        {
                            self.db.hostname,
                            'TXT',
                            'platform_info: ' ..
                                cjson.encode(self.db.platform_info)
                        },
                    })
            end
        end, 5, 1):start(loop)

    busobj.NameOwnerChanged:bind(function(...) self:name_owner_changed(...) end)

    self.multifd, self.mr = self:make_server('ff02::ffff:1%' .. int_if, 0xd15c)
    assert(unix.setsockopt(self.multifd, unix.IPPROTO_IPV6,
        unix.IPV6_JOIN_GROUP, self:make_ipv6_mreq('ff02::ffff:1', int_index)))

    self.unifd = self:make_server(self.int_ip, 0xd15c)

    self.tcpserv = net.socket{ip = self.int_ip, port = 0xd15c}:listen()
    self.tcpserv.acceptor = function(...) self:hadv_acceptor(...) end
    self.tcpserv:start(loop)

    self.second_timer = ev.Timer.new(function() self:periodic() end, 0.01, 1)
    self.second_timer:start(loop)

    return self
end

--! D-Bus signal sent whenever the name database changes.
--!
--! \param removed_rrs The RRs which were removed in the change.
--! \param added_rrs The RRs which were added in the change.
--? signal Network.ChangedRRs removed_rrs added_rrs
Network_iface:add_signal{'ChangedRRs', 'a(sssss)a(sssss)'}

function Network:changed_rrs(del, add)
    self:ChangedRRs(del, add)
end

function Network:local_deltas(deltas)
    self:send_hadv(deltas)
end

function Network:name_owner_changed(name, old, new)
    if name == old and new == '' then
        local ok, err = pcall(self.db.delete_local_source, self.db, old)
        if not ok then
            local retry = 1
            ev.Timer.new(function(l, t, e)
                    retry = retry + 1
                    if retry > 12 then
                        log:error("Unable to perform name_owner_changed")
                        os.exit(1)
                    end
                    local ok, err =
                        pcall(self.db.delete_local_source, self.db, old)
                    if ok then
                        t:stop(l)
                        t:clear_pending(l)
                        return
                    end
                end, .25, .25):start(loop)
        end
    end
end

function errwrap(err, desc, val, ...)
    if val then
        return val, ...
    else
        return nil, err, desc
    end
end

--! Resolves a network name to its canonical form.
function Network:ResolveName(name)
    return errwrap('com.somat.armada.Network.NotFound', 'Name not found',
                   self.db:resolve_name(name))
end
Network_iface:add_method{'ResolveName', 's', 's'}

--! Resolves a hostname to an address.
function Network:ResolveHostname(name)
    return errwrap('com.somat.armada.Network.NotFound', 'Hostname not found',
                   self.db:resolve_hostname(name))
end
Network_iface:add_method{'ResolveHostname', 's', 'ss'}

--! Resolves a service name to an address and port.
function Network:ResolveService(name)
    return errwrap('com.somat.armada.Network.NotFound', 'Service not found',
                   self.db:resolve_service(name))
end
Network_iface:add_method{'ResolveService', 's', 'ssss'}

--! Reverse-resolves an address to a hostname.
function Network:ResolveAddress(addr)
    return errwrap('com.somat.armada.Network.NotFound', 'Address not found',
                   self.db:resolve_address(name))
end
Network_iface:add_method{'ResolveAddress', 's', 's'}

--! Queries the name database.
function Network:QueryRR(owner, type, rdata, source)
    return self.db:query_rr(owner, type, rdata, source)
end
Network_iface:add_method{'QueryRR', 'ssss', 'a(sssss)'}

--! Queries the local name database.
function Network:QueryLocalRR(owner, type, rdata, source)
    return self.db:query_local_rr(owner, type, rdata, source)
end
Network_iface:add_method{'QueryLocalRR', 'ssss', 'a(ssss)'}

--! Modifies the name database.
function Network:ModifyRRs(remove_rrs, add_rrs, reply, msg)
    local sender = msg and msg:get_sender() or bus:bus_get_unique_name()
    self.db:modify_rrs(remove_rrs, add_rrs, sender)
    return true
end
Network_iface:add_method{'ModifyRRs', 'a(sss)a(sss)', ''}

--! Adds a resource record to the name database.
function Network:AddRR(owner, type, rdata, reply, msg)
    return self:ModifyRRs({}, {{owner, type, rdata}}, reply, msg)
end
Network_iface:add_method{'AddRR', 'sss', ''}

--! Removes a resource record from the name database.
function Network:RemoveRR(owner, type, rdata, reply, msg)
    return self:ModifyRRs({{owner, type, rdata}}, {}, reply, msg)
end
Network_iface:add_method{'RemoveRR', 'sss', ''}

--! Removes all resource records from the name database owned by the argument
function Network:RemoveLocalRROwner(owner, reply, msg)
    self.db:remove_local_owner(owner)
    return true
end
Network_iface:add_method{'RemoveLocalRROwner', 's', ''}

--! Removes all resource records from the name database owned by the argument
function Network:RemoveLocalRROTR(owner, type, rdata)
    self.db:remove_local_otr(owner, type, rdata)
    return true
end
Network_iface:add_method{'RemoveLocalRROTR', 'sss', ''}

--! Returns the local module's internal hostname.
function Network:GetModuleHostname()
    return network_config.internal_hostname
end
Network_iface:add_method{'GetModuleHostname', '', 's'}

--! Returns the local module's internal IP address.
function Network:GetModuleIP()
    return network_config.internal_ip
end
Network_iface:add_method{'GetModuleIP', '', 's'}

--! Returns the local module's internal link-local IP address.
function Network:GetModuleLinkLocalIP()
    return network_config.internal_ll_ip
end
Network_iface:add_method{'GetModuleLinkLocalIP', '', 's'}


--[===[
-- FIXME: Remove this when the issue has been resolved on the eDAQXR!
if config.device_type == 'edaqxr' then
local nf = require 'net_functions'
    local ifaces_up = {}
--[[
    -- If a notification should be sent on client connect
    local rr
    local function link_status_callback(iface, state)
        ifaces_up[iface] = state
        if ifaces_up['eth1'] == nil or ifaces_up['eth2'] == nil then
            -- Don't do anything until status' for both network devices has
            -- been reported
            return
        elseif ifaces_up['eth2'] and not ifaces_up['eth1'] then
            -- ETH2 up and ETH1 down is illegal
            rr = {
                '_system-notification-' .. uuid.generate(),
                'TXT',
                cjson.encode({
                    type = 'warning',
                    severity = 'none',
                    msg = "We have detected that ETH2 is connected and " ..
                        "ETH1 is not connected.  This is not a valid way " ..
                        "to use ETH1/ETH2.  ETH2 may only be connected if " ..
                        "ETH1 is also connected.  Tests may not be run in " ..
                        "this state.  Note, this issue will be resolved in " ..
                        "a future firmware release.",
                    title = "Unsupported Network Connections",
                    id = id,
                    buttons = nil,
                })}
            network:ModifyRRs({}, {rr})
        elseif rr then
            network:ModifyRRs({rr}, {})
            rr = nil
        end
    end

    nf.carrier_up('eth1', loop, link_status_callback)
    nf.carrier_up('eth2', loop, link_status_callback)
]]
    nf.carrier_up('eth1', loop, function(iface, state)
            ifaces_up[iface] = state ~= nil and state or false
        end)
    nf.carrier_up('eth2', loop, function(iface, state)
            ifaces_up[iface] = state ~= nil and state or false
        end)

    function Network:IsNetworkTopologyValid()
        if ifaces_up['eth1'] == nil or ifaces_up['eth2'] == nil then
            return 1
        elseif ifaces_up['eth2'] and not ifaces_up['eth1'] then
            return 0
        end
        return 1
    end
    Network_iface:add_method{'IsNetworkTopologyValid', '', 'i'}
end
]===]

function Network:make_ipv6_mreq(addr, iface) --? private
    -- KLUDGE
    local u = unix
    local r =
        assert(u.getaddrinfo(addr, 0, u.AI_NUMERICHOST + u.AI_NUMERICSERV,
                             u.AF_INET6, u.SOCK_STREAM))[1]
    local addr = r.addr:sub(9, 9 + 15) -- KLUDGE!!!
    local iface = struct.pack('i', iface)
    return addr .. iface
end

function Network:make_server(addr, port, type) --? private
    local s = net.socket{ip = addr, port = port, type = unix.SOCK_DGRAM}
    s:bind()
    local function func(loop, io)
        local buf, addr = assert(unix.recvfrom(s.sock, 8192))
        local ip, port = assert(unix.getnameinfo(addr,
            unix.NI_NUMERICHOST + unix.NI_NUMERICSERV))
        -- print('multi: got', buf:len(), 'bytes from', ip, port)
        self:got_multicast(buf, addr, ip, port)
    end
    local io = ev.IO.new(func, s.sock, ev.READ)
    io:start(loop)
    return s.sock, s.addr
end

function Network:hadv_acceptor(tcpserv, loop, sock) --? private
    local sbuf = net.streambuf(sock)
    sbuf:write(deflate(self.db:get_hadv_response()))
    sbuf:shutdown()
    sbuf:start(loop)
end

function Network:send_hadv(deltas) --? private
    local soa = self.db:get_soa(self.int_ip)
    if not self.unifd then -- KLUDGE initialization ordering
        return
    end
    local ver = struct.pack('>i4', soa)
    if deltas then
        local d = deflate(cjson.encode(deltas))
        if #d < 1400 then
            unix.sendto(self.unifd, 'HDif' .. ver .. d, 0, self.mr.addr)
        end
    end
    unix.sendto(self.unifd, 'HAdv' .. ver, 0, self.mr.addr)
end

function Network:delete_host(ip) --? private
    local host = self.hosts[ip]
    if host then
        self.hosts[ip] = nil -- KLUDGE false to preserve hosts ordering?
        if host.streambuf then
            host.streambuf:shutdown()
        end
        self.db:delete_host(ip)
    end
end

function Network:periodic() --? private
    local timeout = 10
    if not config.no_network_master_db then
        local now = loop:update_now()
        local to_delete = {}
        for ip, host in pairs(self.hosts) do
            if host.localhost then
                -- Do nothing
            elseif not host.streambuf and now - host.last > timeout then
log:log(log.NOTICE, string.format("Heartbeats for host %s lost %f seconds", ip, now - host.last))
                to_delete[#to_delete + 1] = ip
            elseif host.req_timeout then
                if now - host.req_timeout > 10 then
log:log(log.NOTICE, string.format("Request for host %s timed out %f seconds", ip, now - host.req_timeout))
                    to_delete[#to_delete + 1] = ip
                end
            end
        end
        for i = 1, #to_delete do
            self:delete_host(to_delete[i])
        end
    end
    self:send_hadv()
end

local P, S, C, Ct = lpeg.P, lpeg.S, lpeg.C, lpeg.Ct
local grammar =
    Ct(Ct(C((1-S'\t')^0) * S'\t' *
          C((1-S'\t')^0) * S'\t' *
          C((1-S'\n')^0) * S'\n')^1)

function Network.hadv_close_cb(self, loop, dir) -- self is streambuf
    local network = self.network
    local host = network.hosts[self.host_ip]
    local buf = inflate(tostring(self.readbuf))
    local rrs = grammar:match(buf)

    if host then host.req_timeout = nil end
    if not host or not rrs then -- Host went down in the meantime
        network:delete_host(self.host_ip)
        return
    end
    local new_soa = network.db:update_host_rrs(self.host_ip, rrs)
    if not new_soa then
        log:warn(
            __'Ignoring HAdv protocol error, no SOA entry from {0} SOA {1}',
            self.host_ip, self.advertised_soa)
    end
    host.streambuf = nil

    -- Reset the timeouts for all host because this may have taken long
    -- enough to exceed 10 second heartbeat timeout.
    local new_now = loop:update_now()
    for k, v in pairs(network.hosts) do
        network.hosts[k].last = new_now
    end

    self:shutdown()
end

function Network:got_multicast(buf, addr, ip, port)
    local host = self.hosts[ip]
    if not host then
        host = { }
        self.hosts[ip] = host
    end
    if host.streambuf then return end
    host.last = loop:update_now()
    if config.no_network_master_db then
        return
    end
    local type = buf:sub(1, 4)
    local cur_soa = self.db:get_soa(ip)
    if type == 'HDif' then
        local soa = struct.unpack('>i4', buf:sub(5, 8))
        -- Only do the delta if we're one SOA behind and no TCP RR
        -- fetch in progress, otherwise fall back to HAdv behavior
        if not cur_soa or cur_soa + 1 ~= soa or host.streambuf then
            self:process_hadv(host, ip, cur_soa, soa)
            return
        end
        local json = inflate(buf:sub(9))
        local delta = cjson.decode(json)
        if not delta or not delta.del or not delta.add then
            log:warn(__'Ignoring HDif protocol error from {0} bad JSON {1}',
                ip, json)
            self:process_hadv(host, ip, cur_soa, soa)
            return
        end
        self.db:update_host_delta(ip, soa, delta)
    elseif type == 'HAdv' then
        local soa = struct.unpack('>i4', buf:sub(5, 8))
        self:process_hadv(host, ip, cur_soa, soa)
    end
end

function Network:process_hadv(host, ip, cur_soa, soa)
    if cur_soa == soa or host.streambuf then
        return
    end
    local s1 = net.socket{ip = ip, port = 0xd15c}
    local e, s = pcall(s1.connect, s1)
    if not e then
        self:delete_host(ip)
        return
    end
    local sbuf = net.streambuf(s.sock)
    sbuf.host_ip = ip
    sbuf.advertised_soa = soa
    sbuf.network = self
    sbuf.close_cb = self.hadv_close_cb
    host.streambuf = sbuf
    host.req_timeout = ev.Loop.default:update_now()
    sbuf:start(loop)
end

local network = Network.new('/var/lib/armada/network.db')

bus:export_object(network, '/com/somat/armada/Network', { Network_iface })
assert(bus:bus_request_name('com.somat.armada.Network'))

ev.Idle.new(function(l, i, e)
        local local_info = {
            hostname = network_config.internal_hostname,
            internal_ip = network_config.internal_ip,
            internal_ll_ip = network_config.internal_ll_ip,
        }
        network:ModifyRRs({},
            {{'_armada-device-info', 'TXT', cjson.encode(local_info)}})

        -- If development overrides are in place, let client know
        if config.development_overrides and
            config.device_type ~= 'dx23r' then
            local id = uuid.generate()
            network:ModifyRRs({}, {{
                '_system-notification-' .. id,
                'TXT',
                cjson.encode({
                    type = 'notice',
                    severity = 'none',
                    msg = "The 'system_config.lua' file for development " ..
                        "only use is active.",
                    title = "Development Options In Use",
                    id = id,
                    buttons = nil,
                })}})

        end

        i:stop(l)
    end):start(loop)

loop:loop()
