--[[
	Lua module file for translating sdb sensor parameters to 
	cx useable format.
--]]


local t = {}

local sxr_setup = require "sxr_setup"
local supported_sensors = require "translation_rules"
local sensor_sort = require 'sensor_sort'
local log = require "log"
function t:run( sdbx_rows)
	local sxr_rows ,run_info = self:translate( sdbx_rows)
	run_info.blob = sensor_sort:run( run_info.groups ,sxr_rows)
	return run_info
end



function t:translate( sdbx_rows)
	local groups , senpara = {} ,{} --tables for the Group rows and SenPara rows
	local max_group_index = 0 --highest group index found in the rows
	local run_info = { passed=0 ,failed=0}

	 --divide the SenPara and Group row entries
	for row , db_table in pairs( sdbx_rows) do
		if row.PositionIndex then
			--avoid a position index of 0, which we use to store sorting metadata
			row.PositionIndex = tonumber(row.PositionIndex) + 1
		end

		if db_table == "Groups" then
			groups[ row.GroupIndex ] = row
			if row.GroupIndex > max_group_index then
				max_group_index = row.GroupIndex
			end

		elseif db_table == "SenPara" then
			table.insert( senpara , row )
		elseif db_table == 'About' then
			run_info.about = row
		end
	end
	
	--add in null groups for ipair looping (must be continuous to keep ordering )
	for i = 1, max_group_index do
		if groups[ i ] == nil then
			groups[ i ] = { GroupIndex=i , Name="Missing Group" 
				,ParentIndex = 0 , PositionIndex=100 }
		end
	end

	--Actual Translation
	local sxr_rows = {}
	for i , row in ipairs( senpara ) do --finally process every row and translate to sxr_rows
		local sxr_row = {}
        --depth_print(groups)
		local group = groups[ row.GroupIndex ]
        --log:warn(row.GroupIndex .. "")
        --log:warn(groups[3] .. "")
		local settings = self:find_valid_group( row.GenericType )
		print( "Generic Type: "..row.GenericType ,'Name: '..row.Description)

		local failed = false

		if settings and not failed then
			sxr_row ,failed = self:set_fields( row ,settings)
		else
			failed = true
			print( "Failed to find settings for "..row.Description)
		end

		if not failed then
			run_info.passed = run_info.passed + 1
			sxr_row.group_name = group.Name
			sxr_row.description = '"'..string.match( row.Description , '"(.-)"' )..'"'

			--setup the sxr field database_sensor; see sxr_setup.lua for details-
			--done at the end, since most of the fields are just copied in
			sxr_setup.db_row_setup( sxr_row.database_sensor ,sxr_row ,row)

		else
			run_info.failed = run_info.failed + 1
			sxr_row = {} --reset if failed
			sxr_row.description = '"'..string.match( row.Description , '"(.-)"' ) .. '~"'
			sxr_row._supported_categories = {  }
			sxr_row.group_index = row.GroupIndex
		end

		sxr_row.group_name = group.Name
		sxr_row.position_index = tonumber( row.PositionIndex ) + 1
		table.insert( sxr_rows ,sxr_row)
	end

	run_info.groups = groups
	return sxr_rows ,run_info
end
--===== Helpers ======
	function t:find_valid_group( generic_type)
		for _,v in pairs( supported_sensors ) do
			if v.enum == generic_type then
				return v
			end
		end
	end

	function t:set_fields( row , settings )
		local sxr_row = copy_table( sxr_setup.json_f)
		if settings.elec_units then
		    sxr_row.electrical_units 		= settings.elec_units
        end
		sxr_row.output_units 			= row.EngUnit
		sxr_row.source_physical_units 	= row.EngUnit
		sxr_row.group_index				= row.GroupIndex
		sxr_row.database_sensor.source_version = SELF_VERSION

		sxr_row.calibration[ 1].units = sxr_row.source_physical_units
		sxr_row.calibration[ 3].units = sxr_row.output_units

        local failed = false
		failed = self:set_sensor_scale( sxr_row ,row ,settings )
		failed = ( failed or settings.setup( sxr_row ,row ,failed))

		return sxr_row , failed
	end

	function t:set_sensor_scale( sxr_row , sdbx_row , settings )
		local stype = sdbx_row.ScaleType
		local cal = sxr_row.calibration[1]
		local scale_to_use
		local failed = false

		if stype == 0 and ( settings.name == "StrainGage" or settings.name == "Bridge" ) then
			--if 0, then it's assumed ZeroSpan
			if sdbx_row.GageFactor == 0 then
				scale_to_use = settings.s_types[1]
			else --assume StrainGage and check for valid strings; special case '%' conversion to 'µm/m'
				scale_to_use = settings.s_types[2]
				local w = sxr_row.output_units
				w = w:match( '"(.+)"' ) or w
                if w:match("%%") then
                    sxr_row.output_units = 'µm/m'
                elseif not( w=='µm/m' or w=='m/m' or w=='strain' or w=='microstrain' or
				         w=='µStrain' or w == 'µE' or w== 'mm/m' ) then
					failed = true
					print( "Failed output unit check for " .. sdbx_row.Description..", was "..w.."." )
				end
			end
        elseif stype ~= 0 and stype ~= 1 and stype ~= 2 and stype ~= 32 and stype ~= 64 and stype ~= 256 
            and stype ~= 512 then 
		    sxr_row.calibration[1].mode = "defined_zero_span"
		    sxr_row.calibration[1].electrical_zero = 0.0
		    sxr_row.calibration[1].electrical_span = 1.0
		    sxr_row.calibration[1].physical_span = 1.0
		    sxr_row.calibration[1].units = nil
            failed = false
		else
			for i , scale_type in ipairs( settings.s_types ) do
				for j , enum in ipairs( scale_type.enums ) do
					if stype == enum then
						scale_to_use = scale_type
					end
				end
			end
		end

		if scale_to_use and not failed then
			cal.mode = scale_to_use.sxr_name
			failed = scale_to_use.required( cal , sdbx_row )
		end

		return failed
	end




return t
