require 'middleclass'
local ev = require'ev'
local http_server = require 'http_server'
local ffi = require 'ffi'
local C = ffi.C
local log = require 'log'
local cjson = require 'cjson.safe'
local Frame = require 'test_engine/frame'
local Pipe = require 'test_engine/pipe'
local MinMax = require 'test_engine/min_max'
local tcp_server = require 'net/tcp_server'
local tcp_stream = require 'net/tcp_stream'
local ws_server = require 'net/ws_server'
local config = require 'system_config'
local u = require 'unix'
local notify = require 'sys_notify'
local message_can = require 'message_can'

local sn

local Pipe_attach_pipe = Pipe.attach_pipe
local Pipe_detach_pipe = Pipe.detach_pipe
local Pipe_get_frame = Pipe.get_frame
local Pipe_flush_frames = Pipe.flush_frames
local Frame_delete = Frame.delete
local Frame_create = Frame.create

local ffi_sizeof = ffi.sizeof
local ffi_cast = ffi.cast
local ffi_copy = ffi.copy
local ffi_string = ffi.string
local ffi_new = ffi.new

local htonl = ffi.abi('le') and bit.bswap or bit.tobit

local min_max_data = ffi_new('struct min_max_data[1]')

ffi.cdef[[
typedef struct hbm_stream_packet {
    unsigned long       hdr;
    unsigned long       length;
    unsigned char       data[0];
} hbm_stream_packet_t;

typedef struct hbm_small_stream_packet {
    unsigned long       hdr;
    unsigned char       data[256];
} hbm_small_stream_packet_t;

enum {
    HDR_DATA = 0x10000000,
    HDR_METAINFO = 0x20000000
};

enum {
    METATYPE_JSON = 1,
};

enum {
    WS_BINARY = 2,
};

void *realloc(void *, int);
void free(void *);
int pipe(int pipefd[2]);
int read(int, void *, int);
int fcntl(int, int, ...);
]]

local C_realloc = C.realloc
local C_free = C.free

local ss = class('stream_server')

local jsonrpc_version = '2.0'

local api_version_json = cjson.encode({
    method = 'apiVersion',
    params = {'1.0'},
})

local function dbg_msg(fmt, ...)
    if config.te_debug then
        if config.te_debug.hbm_stream then
            log:log(log.NOTICE, string.format(fmt, ...))
        end
    end
end

-- Function used to send short metadata packets on a websocket
function ss:_send_short_metadata(ws, pkt, id, data)
--print(id, data)
    local length = #data
    pkt[0].hdr = htonl(C.HDR_METAINFO + ((length + 4) * 1048576) + id)
    ffi_cast('unsigned int *', pkt[0].data)[0] = htonl(C.METATYPE_JSON)
    ffi_copy(pkt[0].data + 4, data, length)
    ws:send(pkt, length + 8)
end

-- Function used to send long metadata packets on a websocket
function ss:_send_long_metadata(ws, pkt, id, data)
    local length = #data
    pkt.hdr = htonl(C.HDR_METAINFO + id)
    pkt.length = htonl(length + 4)
    ffi_cast('unsigned int *', pkt.data)[0] = htonl(C.METATYPE_JSON)
    ffi_copy(pkt.data + 4, data, length)
    ws:send(pkt, length + 12)
end

-- Function used to send metadata packets on a websocket
function ss:send_metadata(ws_info, id, data)
    if not data then
        error("send_metadata(): NIL DATA", true)
        return
    end

    dbg_msg("send_metadata(): id %d = %s", tonumber(id), data)

    if #data <= 252 then
        return self:_send_short_metadata(ws_info.ws, ws_info.s_pkt, id, data)
    else
        local len = #data + ffi_sizeof('hbm_stream_packet_t') + 4
        if len > ws_info.pkt_size then
            ws_info.pkt_size = len
            ws_info.pkt = ffi_cast('hbm_stream_packet_t *',
                ffi_new('char[?]', len))
        end
        return self:_send_long_metadata(ws_info.ws, ws_info.pkt, id, data)
    end
end

-- Function used to send the initial set of metadata on a websocket when a
-- client first connects to the websocket
function ss:send_stream_metadata(ws_info)
    local pkt = ws_info.s_pkt
    local ws = ws_info.ws

    -- Send the API version metadata with stream id 0
    self:send_metadata(ws_info, 0, api_version_json)

    -- Send the "init" metadata with stream id 0
    self:send_metadata(ws_info, 0, cjson.encode({
            method = 'init',
            params = {
                streamId = ws_info.id,
                supported = {
                    alive = {timeout = 2},
                },
                commandInterfaces = {
                    ['jsonrpc-http'] = self.http_server_info,
                },
            },
        }))

    local cl = {}
    if not ws.is_tcp then
        for _, name in pairs(self.channel_list) do
            if self.channel_info[name].is_local then cl[#cl + 1] = name end
        end
        for _, v in pairs(self.remote_streams or {}) do
            if v.stream then
                for _, name in ipairs(v.cl or {}) do cl[#cl + 1] = name end
            end
        end
    else
        for _, name in pairs(self.channel_list) do
            -- Only local channels OTHER than elapsed_test_run_time* or
            -- sie_file_size* are available on TCP streams
            if self.channel_info[name].is_local and 
                not name:find('sie_file_size') and not
                name:find('elapsed_test_run_time') then
                cl[#cl + 1] = name
            end
        end
    end
--log:log(log.NOTICE, string.format("Available channels: %s", cjson.encode(cl)))
    self:send_metadata(ws_info, 0, cjson.encode({
            method = 'available',
            params = cl,
        }))
end

-- Function called when a new TCP client connects
function ss:tcp_connect(server, socket)
    local stream = tcp_stream:new(ev.Loop.default, socket)
    stream:on_close(function(...) self:tcp_close(...) end)
    stream.is_tcp = true

    -- Generate the "alive" metadata packet for this stream
    local alive_pkt = ffi_new('hbm_small_stream_packet_t[1]')
    local data = cjson.encode({
        method = 'alive',
    })
    local length = #data
    alive_pkt[0].hdr = htonl(C.HDR_METAINFO + ((length + 4) * 1048576))
    ffi_cast('unsigned int *', alive_pkt[0].data)[0] = htonl(C.METATYPE_JSON)
    ffi_copy(alive_pkt[0].data + 4, data, length)

    local id = string.format("TCP_STREAM_%d", math.random(10000))
    local client_info = {
        ws = stream,
        subscribed_channels = {},
        pkt_size = ffi_sizeof('hbm_stream_packet_t'),
        pkt = ffi_cast('hbm_stream_packet_t *',
            ffi_new('char[?]', ffi_sizeof('hbm_stream_packet_t'))),
        s_pkt = ffi_new('hbm_small_stream_packet_t[1]'),
        id = id,
        fsts_sent = {},
    }
    self.client_list[#self.client_list + 1] = client_info

    -- Timer used to send "alive" metadata messages on the stream
    local hbt = ev.Timer.new(function(l, t, e)
            stream:send(alive_pkt, length + 8)
        end, 1, 1)
    hbt.info = self.client_list[#self.client_list]
    self.client_list[#self.client_list].hbt = hbt

    -- If everything is OK, send the "stream" metadata back on the stream
    self:send_stream_metadata(client_info)

    -- Start the heartbeat timer
    hbt:start(self.loop)
end

-- Function called when a new websocket client connects
function ss:ws_connect(server, socket)
    local stream = tcp_stream:new(ev.Loop.default, socket)
    local ws = ws_server:new(stream)
    ws:on_close(function(...) self:ws_close(...) end)
    ws:on_handshake(function(...)

        self.ws_streams = self.ws_streams or 0
        self.ws_streams = self.ws_streams + 1

        -- Generate the "alive" metadata packet for this stream
        local alive_pkt = ffi_new('hbm_small_stream_packet_t[1]')
        local data = cjson.encode({
            method = 'alive',
        })
        local length = #data
        alive_pkt[0].hdr = htonl(C.HDR_METAINFO + ((length + 4) * 1048576))
        ffi_cast('unsigned int *', alive_pkt[0].data)[0] =
            htonl(C.METATYPE_JSON)
        ffi_copy(alive_pkt[0].data + 4, data, length)

        local id = string.format("WS_STREAM_%d", math.random(10000))
        local client_info = {
            ws = ws,
            subscribed_channels = {},
            pkt_size = ffi_sizeof('hbm_stream_packet_t'),
            pkt = ffi_cast('hbm_stream_packet_t *',
                ffi_new('char[?]', ffi_sizeof('hbm_stream_packet_t'))),
            s_pkt = ffi_new('hbm_small_stream_packet_t[1]'),
            id = id,
            fsts_sent = {},
        }
        self.client_list[#self.client_list + 1] = client_info

        -- Timer used to send "alive" metadata messages on the stream
        local hbt = ev.Timer.new(function(l, t, e)
                ws:send(alive_pkt, length + 8)
            end, 1, 1)
        hbt.info = self.client_list[#self.client_list]
        self.client_list[#self.client_list].hbt = hbt

        -- If everything is OK, send the "stream" metadata back on the stream
        self:send_stream_metadata(client_info)

        -- Start the heartbeat timer
        hbt:start(self.loop)
    end)
end

-- Function called when a TCP client disconnects
function ss:tcp_close(ws)
    -- TCP connection from master closed, there is no need to continue moving
    -- data from local channels to the master
    if self.data_writer then
        self.data_writer:stop(ev.Loop.default)
        self.data_writer:clear_pending(ev.Loop.default)
    end

    -- We don't really need to do anything here since the only time this
    -- happens is when a mode 2 networked test is stopping, so this test slice
    -- is also exiting.
end


-- Function called when a websocket client disconnects
function ss:ws_close(ws)
    -- Close all of the remote streams associated with this stream
    if self.remote_streams and next(self.remote_streams) then
        self.ws_streams = self.ws_streams - 1
        if self.ws_streams < 1 then
            for node, rs in pairs(self.remote_streams) do
                if rs.stream then
                    dbg_msg("    Unsubscribe all channels from node %s", node)
                    local unsubed = rs.stream:remove_all_subscriptions()
                    dbg_msg("    Unsubscribed %s", table.concat(unsubed, ', '))
                end
            end
        end

        -- Remove this websocket streams attachment to any remote stream's
        -- processing loop
        for _, ci in pairs(self.channel_info) do
            if ci.remote_stream and ci.remote_stream.ws and
                ci.remote_stream.ws[ws] then
                ci.remote_stream.ws[ws] = nil
            end
        end
    end

    -- Remove the client information from the client info list
    for i, ci in ipairs(self.client_list) do
        if ci.ws == ws then
            -- Make sure we "unsubscribe" from each of the channels subscribed
            -- on this connection
            for _, subscription in ipairs(ci.subscribed_channels) do
                local cinfo = self.channel_info[subscription.name]

                if cinfo then
                    cinfo.subscribers = cinfo.subscribers - 1
                    if cinfo.subscribers == 0 and cinfo.pipe ~= nil then
                        if cinfo.source_pipe_list then
                            Pipe_detach_pipe(cinfo.source_pipe_list, cinfo.pipe)
                        end
                        cinfo.pipe = nil
                    end
                end
            end

            -- Stop the heartbeats for this client
            if ci.hbt then ci.hbt:stop(self.loop) end
            table.remove(self.client_list, i)
            return
        end
    end
end

-- Function used to get a client list entry by stream id
function ss:get_client(stream_id)
    for _, c in ipairs(self.client_list) do
        if c.id == stream_id then
            return c
        end
    end
end

-- Function used to extract the stream id from the methods used to subscribe or
-- unsubscribe channels on a stream
function ss:get_stream_id(method, req_id)
    local stream_id = method:match('^(.+)%.unsubscribe$') or
        method:match('^(.+)%.subscribe$') or
        method:match('^(.+)%.want_min_max$') or
        method:match('^(.+)%.no_min_max$')
    if stream_id then
        local client = self:get_client(stream_id)
        if not client then
            return nil, cjson.encode({
                jsonrpc = jsonrpc_version,
                error = {
                    message = 'Stream ID not found',
                    code = -32601,
                },
                id = req_id,
            })
        end
        return stream_id
    end
    return nil, cjson.encode({
        jsonrpc = jsonrpc_version,
        error = {
            message = 'Unable to parse stream id from method',
            code = -32603,
        },
        method = method,
        id = req_id,
    })
end

-- Mapping of XR test engine datatypes to HBM streaming protocol datatypes
local value_types = {
    sequential_float32 = 'real32',
    sequential_float64 = 'real64',
    message_float32 = 'real32',
    message_float64 = 'real64',

    -- NOTE:  The entries below are here for completeness but no native data
    -- source on a XR supplies data in any of these formats.
    int32 = 's32',
    uint32 = 'u32',
    int64 = 's64',
    uint64 = 'u64',
}

-- Somat protocol extension: Stream control function called by a client to
-- request that the test engine perform min/max value tracking of a "low
-- speed" channel and send metadata messages containing the min/max values.
-- This extension is intended for "low speed" channels ONLY but this is not
-- enforced by the server (i.e., we support it even for full-speed channels).
-- This feature is consumed by the HBM Champaign stream library and plotter.
function ss:want_min_max(json)
    return self:set_min_max(json, true)
end
function ss:no_min_max(json)
    return self:set_min_max(json, false)
end

function ss:set_min_max(json, value)
    local req_id = json.id
    local stream_id, err = self:get_stream_id(json.method, req_id)
    if not stream_id then
        return err
    end
    local failed_list
    local client = self:get_client(stream_id)
    local subscribed_channels = client.subscribed_channels
    for _, sig_name in ipairs(json.params) do
        local cinfo = nil
        for _, sc in ipairs(subscribed_channels) do
            if sc.name == sig_name then
                cinfo = sc
                break
            end
        end
        if cinfo then
            cinfo.want_min_max = value
        else
            failed_list = failed_list or {}
            failed_list[#failed_list + 1] = sig_name
        end
    end

    -- If the request came in on the WS connection forward it to all of the
    -- remote stream connections.
    if json.method:find('WS_STREAM') then
        for node, rs in pairs(self.remote_streams) do
            rs.stream:set_min_max(json.params, value)
        end
    end

    if failed_list then
        return cjson.encode({
            jsonrpc = jsonrpc_version,
            error = {
                message = 'Invalid params',
                code = -32602,
                data = failed_list,
            },
            id = req_id,
        })
    end
    return cjson.encode({
        jsonrpc = jsonrpc_version,
        result = {},
        id = req_id,
    })
end

-- Function called to process an incoming subscribe request for a stream
-- FIXME:  Does subscribing to a channel that has already been subscribed to
-- result in an error response or success?  And, if success, is a second
-- instance provided in the stream or is the existing instance used?
function ss:subscribe(json)
    local req_id = json.id
    local stream_id, err = self:get_stream_id(json.method, req_id)
    if not stream_id then
        return err
    end
    local failed_list
    local client = self:get_client(stream_id)
    local ws = client.ws
    local ws_send = ws.send
    local subscribed_channels = client.subscribed_channels
    local pkt = client.s_pkt
    local rs = {}
    for _, sig_name in ipairs(json.params) do
        -- Check to see if the channel has already been subscribed to
        local cinfo
        local is_remote
        for s, ci in pairs(self.channel_info) do
            if s == sig_name then
                if not ci.subscribers or ci.subscribers == 0 then
                    if ci.source_pipe_list then
--[[
log:log(log.NOTICE, string.format("Adding subscription to local '%s'",
    sig_name))
]]
                        ci.pipe = Pipe_attach_pipe(ci.source_pipe_list)
                    elseif ci.remote_stream then
--[[
log:log(log.NOTICE, string.format("Adding subscription to remote '%s'",
    sig_name))
]]
--                        is_remote = true
                        if not ci.remote_stream.stream then
                            return cjson.encode({
                                jsonrpc = jsonrpc_version,
                                error = {
                                    message = 'Invalid params',
                                    code = -32602,
                                    data = {sig_name},
                                },
                                id = req_id,
                            })
                        end
                    end
                    ci.subscribers = ci.subscribers or 0
                end
                if ci.remote_stream then
                    is_remote = true
                    ws.rs_list = ws.rs_list or {}
                    ws.rs_list[ci.remote_stream] = true
                    ws.si = ws.si or {}
                    ws.si[ci.slot] = 3
                    ci.remote_stream.ws[ws] = true
                    dbg_msg(
                        "Proxying remote channel '%s' (%d) from %s on %s",
                        sig_name, ci.slot,
                        ci.remote_stream.stream:get_stream_id(), stream_id)
                    ci.remote_stream.stream:add_subscription(sig_name,
                        ci.remote_stream.out_fd)
                    rs[ci.remote_stream.stream] = true
                end
                ci.subscribers = ci.subscribers + 1
                cinfo = ci
                break
            end
        end

        if cinfo then
            if not is_remote then
                -- Send the subscribe metadata for this channel
                -- on the websocket client
                self:send_metadata(client, cinfo.slot, cjson.encode({
                        method = 'subscribe',
                        params = {sig_name},
                    }))

                -- Send the signal rate metadata for this channel
                -- on the websocket client.
                -- FIXME:  What should be done here for "asynchronous" channels?
                if cinfo.md.sample_rate and cinfo.md.sample_rate > 0 then
                    self:send_metadata(client, cinfo.slot, cjson.encode({
                            method = 'signalRate',
                            params = {
                                samples = cinfo.md.sample_rate,
                                delta = {
                                    type = 'ntp',
                                    seconds = 1,
                                    fraction = 0,
                                },
                            },
                        }))
                end
    
                -- Send the signal "data" metadata for this
                -- channel on the websocket client
                self:send_metadata(client, cinfo.slot, cjson.encode({
                        method = 'data',
                        params = {
                            pattern =
                                -- Numeric message, type is TV
                                (cinfo.md.data_type:find('^message_float') and
                                    'TV') or
                                -- Other numeric type (sequential), type is V
                                (cinfo.md.data_type:find('_float') and 'V') or
                                -- All other data types (CAN, video, raw GPS,
                                -- and utc_date_time), type is TB
                                'TB',
                            endian = ffi.abi('le') and 'little' or 'big',
                            valueType =
                                -- If message_format exists use it as the
                                -- valueType.
                                (cinfo.md.message_format and
                                    cinfo.md.message_format:lower()) or
                                -- Otherwise look it up based on data type
                                value_types[ffi_string(cinfo.md.data_type)],
                            time = {
                                type = 'ntp',
                                size = 8,
                            },
                        },
                    }))
            end

            -- Reset to allow fsts to be sent again
            client.fsts_sent[sig_name] = nil

            -- Tell the system to start sending data for this channel on the
            -- appropriate websocket client.  Make sure data sent is limited to
            -- once per subscribed channel!
            local found
            for _, sc in ipairs(subscribed_channels) do
                if sc.name == sig_name then
                    found = true
                    break
                end
            end
            if not found then
                subscribed_channels[#subscribed_channels + 1] = {
                    name = sig_name,
                    cinfo = cinfo,
                    want_min_max = false
                }
            end
        else
            failed_list = failed_list or {}
            failed_list[#failed_list + 1] = sig_name
        end
    end
    if failed_list then
        return cjson.encode({
            jsonrpc = jsonrpc_version,
            error = {
                message = 'Invalid params',
                code = -32602,
                data = failed_list,
            },
            id = req_id,
        })
    end
    for stream, _ in pairs(rs) do
        self.sub_timer = self.sub_timer or {}
        if stream:channels_available() > 0 and
            not self.sub_timer[stream] then
            stream:update_subscriptions(self.current_id)
        elseif not self.sub_timer[stream] then
            self.sub_timer[stream] = ev.Timer.new(function(l, t, e)
                    if stream:channels_available() > 0 then
                        t:stop(l)
                        stream:update_subscriptions()
                        self.sub_timer[stream] = nil
                    end
                end, .1, .1)
            self.sub_timer[stream]:start(ev.Loop.default)
        end
    end
    return cjson.encode({
        jsonrpc = jsonrpc_version,
        result = {},
        id = req_id,
    })
end

function ss:get_subscribed_channel_by_name(subscribed_channels, sig_name)
    for i, info in ipairs(subscribed_channels) do
        if info.name == sig_name then
            return subscribed_channels[i], i
        end
    end
end

-- Function called to process an incoming unsubscribe request for a stream
function ss:unsubscribe(json)
    local req_id = json.id
    local stream_id, err = self:get_stream_id(json.method, req_id)
    if not stream_id then
        return err
    end
    local failed_list
    local client = self:get_client(stream_id)
    local ws = client.ws
    local ws_send = ws.send
    local pkt = client.s_pkt
    local subscribed_channels = client.subscribed_channels
    local rs = {}
    for _, sig_name in ipairs(json.params) do
        local e, i =
            self:get_subscribed_channel_by_name(subscribed_channels, sig_name)
        if e and i then
            local cinfo = self.channel_info[sig_name]

            -- Detach the pipe from the source if all have unsubscribed
            -- more clients using the channel
            cinfo.subscribers = cinfo.subscribers - 1
            if cinfo.subscribers == 0 then
                if cinfo.source_pipe_list then
                    Pipe_detach_pipe(cinfo.source_pipe_list, cinfo.pipe)
                elseif cinfo.remote_stream then
                    cinfo.remote_stream.stream:remove_subscription(sig_name)
                    rs[cinfo.remote_stream.stream] = true
                end
                cinfo.pipe = nil
            end

            -- Send the unsubscribe metadata for this channel on the websocket
            -- client
            self:send_metadata(client, cinfo.slot, cjson.encode({
                    method = 'unsubscribe',
                }))

            -- Re-arm sending of first sample timestamp info if channel is
            -- resubscribed on this client
            client.fsts_sent[sig_name] = nil

            -- Tell the system to start sending data for this channel on the
            -- appropriate websocket client
            table.remove(subscribed_channels, i)
        else
            failed_list = failed_list or {}
            failed_list[#failed_list + 1] = sig_name
        end
    end
    if failed_list then
        return cjson.encode({
            jsonrpc = jsonrpc_version,
            error = {
                message = 'Invalid params',
                code = -32602,
                data = failed_list,
            },
            id = req_id,
        })
    end
    for stream, _ in pairs(rs) do stream:update_subscriptions() end
    return cjson.encode({
        jsonrpc = jsonrpc_version,
        result = {},
        id = req_id,
    })
end

-- Handler for HTTP requests coming in from the stream control server
function ss.http_handler(loop, w, http)
    local self = w.me
    if not http.uri:match('^' .. self.http_server_info.httpPath) then
        w.streambuf:write("HTTP/1.1 404 NOT FOUND\r\n\r\n")
        w.server.http_write_shutdown(loop, w)
        return
    elseif http.method ~= self.http_server_info.httpMethod then
        w.streambuf:write("HTTP/1.1 400 INVALID METHOD\r\n\r\n")
        w.server.http_write_shutdown(loop, w)
        return
--[[
    elseif not http.headers or not http.headers.content_type or
        not http.headers.content_type:find('application/json') then
        w.streambuf:write("HTTP/1.1 400 INVALID CONTENT TYPE\r\n\r\n")
        w.server.http_write_shutdown(loop, w)
        return
]]
    elseif not http.content then
        w.streambuf:write("HTTP/1.1 400 EMPTY CONTENT\r\n\r\n")
        w.server.http_write_shutdown(loop, w)
        return
    end
    local cj = cjson.decode(http.content)
    if not cj then
        w.streambuf:write("HTTP/1.1 400 UNABLE TO DECODE JSON\r\n\r\n")
        w.server.http_write_shutdown(loop, w)
        return
    elseif not cj.jsonrpc or cj.jsonrpc ~= jsonrpc_version then
        w.streambuf:write("HTTP/1.1 400 INVALID RPC VERSION\r\n\r\n")
        w.server.http_write_shutdown(loop, w)
        return
    end
    local method = cj.method:match('^.+%.(.+)$')
    if not method or not self[method] then
        w.streambuf:write("HTTP/1.1 400 UNRECOGNIZED REQUEST\r\n\r\n")
        w.server.http_write_shutdown(loop, w)
        return
    end
    local json = self[method](self, cj)
    w.streambuf:write(
        "HTTP/1.1 200 OK\r\n" ..
        "Content-Length: " .. #json + 2 .. "\r\n" ..
        "Content-Type:  application/json\r\n\r\n" ..
        json .. "\r\n")
    w.server.http_write_shutdown(loop, w)
end

local function get_zcip_address(host)
end

function ss:connect_remotes()
    for node, v in pairs(self.remote_streams or {}) do
        if not self.remote_streams[node].stream then
            local ip = require('network_db'):new():get_zcip_address(node)
            self.remote_streams[node].stream =
                require("mx.hbmstream"):new(nil, ip, "1081", "8081")
            if self.remote_streams[node].stream.stream == nil then
                -- error
            end
            self.remote_streams[node].stream:set_error_handler(
                function(_msg, errno)
                    local msg = ffi.string(_msg)
                    if not msg:find('write to pipe') then return 0
                    else
                        local sn = notify:new('Chart Displays Deactivated')
                        sn:warning("One or more chart displays have " ..
                            "been deactivated.  This occurs when the " ..
                            "system is close to being overloaded.  " ..
                            "You can refresh the browser to reactivate " ..
                            "the affected chart display(s).")
                        -- TODO: notification indicating loss of chart data
                        log:log(log.WARNING,
                            "HBM stream interconnect pipe closed")
                    end
                    return 1
                end)
            -- Make the channels on this stream available on
            -- ALL of the websocket client streams
            for _, cinfo in ipairs(self.client_list) do
                self:send_metadata(cinfo, 0, cjson.encode({
                        method = 'available',
                        params = self.remote_streams[node].cl,
                    }))
            end
        end
    end
end

local function decode_buf(ws, buf, avail)
    local offset = 0
    local _b = ffi.new('unsigned char *', buf)

    while offset < avail do
        local b = _b + offset
        local type = tonumber(bit.band(bit.rshift(b[0], 4), 0x3))
        local size = tonumber(bit.band(bit.lshift(b[0], 4), 0xf0) +
            bit.band(bit.rshift(b[1], 4), 0xf))
        local signo = tonumber(bit.lshift(bit.band(b[1], 0xf), 16) +
            bit.band(bit.lshift(b[2], 8), 0xff00) +
            bit.band(b[3], 0xff))
        local n = 4;
        if size == 0 then
            if offset + 8 >= avail then
                log:log(log.NOTICE, "ARG!! Don't have full long packet")
                return
            end
            size = tonumber(bit.band(bit.lshift(b[4], 24), 0xff000000) +
                bit.band(bit.lshift(b[5], 16), 0x00ff0000) +
                bit.band(bit.lshift(b[6], 8), 0x0000ff00) +
                bit.band(b[7], 0x000000ff))
            n = 8;
        end
        if n + size + offset > avail then
            log:log(log.NOTICE, "ARG!! Don't have full packet!")
            return
        end

        if config.te_debug then
            if config.te_debug.hbm_stream_detailed then
                log:log(log.NOTICE,
                    string.format("type: %d, size: %d, signo: %d",
                    type, size, signo))
            end
            if type == 2 then
                dbg_msg("Meta data from remote for signo %d: %s",
                    signo, ffi.string(b + 4 + n, size - 4))
            end
        end

        offset = offset + n + size

        if type == 2 then
            -- Only specific metadata gets forwarded to WS connections
            if signo == 0 then
                local str = ffi.string(b + 4 + n, size - 4)
                -- Only signal number 0 "available" metadata is forwarded.
                if str:find('method":"available') then
                    ws:send(b, n + size)
                end
            elseif ws.si[signo] and ws.si[signo] > 0 then
                -- Metadata for specific signal numbers is forwarded only if
                -- it is expected on a particular WS connection
                ws:send(b, n + size)
                -- TB data type channels don't send signalRate metadata so it
                -- isn't counted when decrementing the metadata counter.
                if not ffi.string(b + 4 + n, size - 4):find("signalRate") then
                    ws.si[signo] = ws.si[signo] - 1
                end
            else
                local str = ffi.string(b + 4 + n, size - 4)
                if str:find('method":"min_max') then
                    ws:send(b, n + size)
                end
            end
        elseif signo > 0 and ws.si[signo] and ws.si[signo] == 0 then
            -- Data for specific signal numbers is forwarded only when it is
            -- expected on a particular WS connection
            ws:send(b, n + size)
        end

    end
end

function ss:process_remote_stream(stream_info)
    local amt_read = C.read(stream_info.in_fd, stream_info.buf,
        ffi.sizeof(stream_info.buf))
--log:log(log.NOTICE, string.format("Got %d bytes of data", amt_read))
    if amt_read > 0 then
        -- Push everything read out to the WS clients. Attached to this remote
        -- stream.
        for ws, _ in pairs(stream_info.ws) do
--log:log(log.NOTICE, "Push to WS stream " .. tostring(ws))
            -- The line below should work but doesn't because the GUI seems to
            -- have issues with receipt of data than consists of more than one
            -- HBM stream protocol packet.  Debugging in the GUI shows that
            -- between each of the HBM stream protocol packets there is some
            -- unknown data that it doesn't understand.  That data looks a lot
            -- to me like the HBM stream protocol header for the subsequent
            -- HBM stream protocol packet!
--            ws:send(stream_info.buf, amt_read)
            decode_buf(ws, stream_info.buf, amt_read)
        end
    else
    end
    return amt_read
end

-- Function used to add a channel to the servers list of channels available to
-- be served on the websocket connections.
function ss:add_channel(md, pipe_list, sid)
    -- The pipe list is nil this is a remote channel so create a node specific
    -- entry in a table of stream connections to remote nodes.
    local node
    self.remote_streams = self.remote_streams or {}
    if not pipe_list then
        if md.category and md.category:find('^filter/') then
            node = md.network_node
        elseif md.connector and md.category and
            md.category:find('^source/') then
            node = md.connector:match('%.(cx23r%-.+)$') or
                md.connector:match('%.(edaqx.-%-.+)$') or
                (md.network_node and
                    md.network_node:match('^(cx23r%-.+)$') or
                    md.network_node:match('^(edaqx.-%-.+)$'))
        end
        if node then
            if not self.remote_streams[node] then
                local p = ffi.new('int[2]')
                C.pipe(p)

                -- Increase the pipe size to max
                if config.device_type ~= 'cx23r' then
                    C.fcntl(p[1], C.F_SETPIPE_SZ, ffi.new('int', 1024 * 1024))
                end

                -- Make the write end of the pipe non-blocking
                u.setfl(p[1], u.O_NONBLOCK)

--log:log(log.NOTICE, string.format("New remote node '%s'", node))
                self.remote_streams[node] = {
                    out_fd = p[1],
                    in_fd = p[0],
                    cl = {},
                    buf = ffi.new('unsigned char[1<<18]'),
                    buf_offset = 0,
                    ws = {},
                    watcher = ev.IO.new(function(l, i, e)
                        if self:process_remote_stream(self.remote_streams[node])
                            <= 0 then
                            C.close(p[0])
                            i:stop(l)
                        end
                    end, p[0], ev.READ)
                }
                self.remote_streams[node].watcher:start(ev.Loop.default)
            end
            self.remote_streams[node].cl[#self.remote_streams[node].cl + 1] =
                md.name
            if md.data_type:find("_float%d+$") then
                -- If this is a numeric data channel, add a low speed version
                self.remote_streams[node].cl[
                    #self.remote_streams[node].cl + 1] = md.name .. '_low_speed'
            end
--[[
log:log(log.NOTICE, string.format("Channel '%s' on remote node '%s'",
    md.name, node))
]]
        else
            -- Appears to be remote channel but does does not have a
            -- have "network_node" in the stanza!
        end
    end

    if not self.channel_info[md.name] then
        if md.sample_rate then md.sample_rate = tonumber(md.sample_rate) end
        local sid
        if md.sid then
            sid = md.sid
        else
            sid = self.max_sid + 1
        end
        if sid > self.max_sid then self.max_sid = sid end

        if config.te_debug and config.te_debug.hbm_stream_detailed then
            log:log(log.NOTICE, string.format("Channel '%s' SID: %d",
                md.name, tostring(sid)))
        end
        self.channel_list[sid] = md.name
        self.channel_info[md.name] = {
            source_pipe_list = pipe_list,
            is_local = pipe_list ~= nil,
            slot = sid,
            md = md,
            is_async = md.data_type:find('^message_') and true,
            message_format = md.message_format,
            remote_stream = pipe_list == nil and self.remote_streams[node],
        }

        sid = sid + 1
        if md.data_type:find("_float%d+$") then
            -- If this is a numeric data channel, add a low speed version of it
            if sid > self.max_sid then self.max_sid = sid end
            if config.te_debug and config.te_debug.hbm_stream_detailed then
                log:log(log.NOTICE,
                    string.format("Channel '%s_low_speed' SID: %d",
                    md.name, tostring(sid)))
            end
            local ls_cname = md.name .. '_low_speed'
            self.channel_list[sid] = ls_cname
            self.channel_info[ls_cname] = {
                source_pipe_list = pipe_list,
                is_local = pipe_list ~= nil,
                slot = sid,
                md = md,
                is_low_speed = true,
                remote_stream = pipe_list == nil and self.remote_streams[node],
            }
        end
    end
    return self.channel_info[md.name].slot
end

-- Stuff needed to make sure we don't end up with unaligned accesses
local ts_ptr = ffi.new('int64_t[1]')
local s_ptr = ffi.new('int32_t[1]')
local f_ptr = ffi.new('uint32_t[1]')

local _spl_buf = ffi.new('char[1000000]')
local _spl = ffi_sizeof('hbm_stream_packet_t')
local _pkt = ffi_cast('hbm_stream_packet_t *', _spl_buf)
local ts_size = ffi.sizeof('int64_t')
local int_size = ffi.sizeof('int32_t')
local mc_size = ffi.sizeof('message_can')

-- Prevent all of the stuff above from getting garbage collected!
-- Sometimes I really dislike Lua!
ss.nc_table = {}
ss.nc_table[ts_ptr] = true
ss.nc_table[s_ptr] = true
ss.nc_table[f_ptr] = true
ss.nc_table[_spl_buf] = true
ss.nc_table[_spl] = true
ss.nc_table[_pkt] = true
ss.nc_table[ts_size] = true
ss.nc_table[int_size] = true
ss.nc_table[mc_size] = true

local function ts_to_ntp(p, pkt)
    ffi.copy(ts_ptr, p, ts_size)
    local ts = tonumber(ts_ptr[0]) / 40000000.0
    local s, f = math.modf(ts)
    s_ptr[0], f_ptr[0] = s, math.floor(f * (2^32))
    ffi.copy(pkt.data, s_ptr, int_size)
    ffi.copy(pkt.data + int_size, f_ptr, int_size)
end

-- Function used to send CAN message data on the HBM stream.  This function
-- demultiples CAN message data packets from a frame and sends each as
-- individual HBM stream packets.
function ss:process_can_message_frame(frame, sid, cinfo)
    -- Get a pointer to the data in the frame
    local p = ffi_cast('char *', frame.buf)

    -- Process all of the data from the frame
    for i = 1, frame.count do
        -- Set the length.  This is the data length as encoded in the 13th
        -- byte of the CAN message packet PLUS the size of the message id.
        local len = bit.band(p[mc_size - 1], 0xf) + ffi.sizeof('uint32_t')

        -- Set the stream ID in the header
        _pkt[0].hdr = htonl(C.HDR_DATA + sid)
        -- Set the length (8 bytes for ts + 13 bytes of data)
        _pkt[0].length = htonl(len + ts_size)

        -- Copy the data to the packet.  Note, the packet data comes after the
        -- NTP timestamp which takes the first 8 bytes.
        ffi.copy(_pkt.data + ts_size, p + ts_size, len);

        -- Convert the time stamp from our ts format to NTP time stamp format
        ts_to_ntp(p, _pkt)

        -- Increment the pointer to the next entry in the frame.
        p = p + mc_size;

        -- Send the packet on the connection
        cinfo.ws:send(_pkt, len + ts_size + 8)
    end
end

-- Function used to send raw message data on the HBM stream.
function ss:process_raw_message_frame(frame, sid, cinfo)
    -- Get a pointer to the data in the frame
    local p = ffi_cast('char *', frame.buf)

    -- Set the length
    local len = ffi.cast('unsigned int *', p)[0] - ts_size

    -- If the length is too large, drop the frame on the floor
    if len > (ffi.sizeof(_spl_buf) - (ts_size + 8)) then
        log:log(log.WARNING, "Raw message is too large for stream")
        return
    end

    -- Step past the length embedded in the raw message
    p = p + 4

    -- Set the stream ID in the header
    _pkt[0].hdr = htonl(C.HDR_DATA + sid)
    -- Set the length (8 bytes for ts + len of data)
    _pkt[0].length = htonl(len + ts_size)

    -- Copy the data to the packet.  Note, the packet data comes after the
    -- NTP timestamp which takes the first 8 bytes.
    ffi.copy(_pkt.data + ts_size, p + ts_size, len);

    -- Convert the time stamp from our ts format to NTP time stamp format
    ts_to_ntp(p, _pkt)

    -- Send the packet on the connection
    cinfo.ws:send(_pkt, len + ts_size + 8)
end

-- Function used to send data from channels to each of the subscribed channels
-- on each of the websocket connections.
function ss:process_streams()
    -- Iterate through all of the clients
    local spl = ffi_sizeof('hbm_stream_packet_t')
    local processed_pipes = {}
    for cid, cinfo in ipairs(self.client_list) do
        -- Iterate through all of the subscribed channels for the client
        -- FIXME:  Maybe make this a function to help the compiler out a bit
        for _, sig_info in ipairs(cinfo.subscribed_channels) do
            local new_frame
            -- Send all available data for this channel on the current client
            -- websocket connection.
            local chan_info = self.channel_info[sig_info.name]
            local is_async = chan_info.is_async
            local pipe = chan_info.pipe
            if pipe ~= nil then
                local frame
                local sid = chan_info.slot
                if not chan_info.is_low_speed then
                    -- Normal channels get all of the samples
                    frame = pipe.last_frame
                else
                    -- Low speed channels only get the last sample from the
                    -- last frame
                    frame = pipe.frame_list
                    if frame ~= nil then
                        new_frame = Frame_create(1, frame.element_size)
                        ffi_copy(new_frame.buf, ffi_cast('char *', frame.buf) +
                            ((frame.count - 1) * frame.element_size),
                            frame.element_size)
                        new_frame.count = 1
                        new_frame.hdr.ts = frame.hdr.ts
                        frame = new_frame
                    end
                end

                local tracked_min, tracked_max
                local have_min_max = false

                if sig_info.want_min_max then
                    if chan_info.md.data_type == 'sequential_float32' then
                        if MinMax.calculate_min_max_float32(
                                pipe, min_max_data) == 0 then
                            tracked_min = min_max_data[0].minimum
                            tracked_max = min_max_data[0].maximum
                            have_min_max = true
                        end
                    elseif chan_info.md.data_type == 'sequential_float64' then
                        if MinMax.calculate_min_max_float64(
                                pipe, min_max_data) == 0 then
                            tracked_min = min_max_data[0].minimum
                            tracked_max = min_max_data[0].maximum
                            have_min_max = true
                        end
                    end
                end

                -- Process all of the frames
                while frame ~= nil do
                    processed_pipes[sig_info.name] = pipe

                    -- If the first sample timestamp metadata has not been sent
                    -- for this channel on this client, send it
                    if not cinfo.fsts_sent[sig_info.name] and
                        tonumber(frame.hdr.ts) then
                        local fsts
                        if not is_async then
                            fsts = tonumber(frame.hdr.ts) /
                                tonumber(chan_info.md.sample_rate)
                        else
                            -- Use the timestamp from the first sample in the
                            -- frame
                            fsts = tonumber(ffi_cast('int64_t *', frame.buf)[0])
                            -- KLUDGE - 40MHz hardcoded!
                            fsts = fsts / 40000000
                        end
                        local s, f = math.modf(fsts)
                        self:send_metadata(cinfo, sid, cjson.encode({
                                method = 'time',
                                params = {
                                    stamp = {
                                        type = 'ntp',
                                        seconds = s,
                                        fraction = math.floor((f * (2^32))),
                                    },
                                },
                            }))
                        cinfo.fsts_sent[sig_info.name] = true
                    end

                    -- Send the frame if the 1st sample time stamp has been
                    -- sent
                    local ts
                    if cinfo.fsts_sent[sig_info.name] then
                        -- Async channel AND not the first frame sent
                        if chan_info.message_format and
                            chan_info.message_format == 'CAN' then
                            self:process_can_message_frame(frame, sid, cinfo)
                            ts = frame.hdr.ts
                        elseif chan_info.message_format and
                            (chan_info.message_format == 'text/nmea' or
                            chan_info.message_format == 'text/plain' or
                            chan_info.message_format == 'image/jpeg') then
                            self:process_raw_message_frame(frame, sid, cinfo)
                            ts = frame.hdr.ts
                        elseif is_async and not new_frame then
                            -- Make a copy of the frame
                            new_frame = Frame_create(frame.count,
                                frame.element_size)
                            ffi_copy(new_frame.buf, frame.buf,
                                frame.element_size * frame.count)
                            new_frame.count = frame.count
                            new_frame.hdr.ts = frame.hdr.ts

                            -- Convert the timestamps in the frame entries to
                            -- NTP time stamps
                            local p = ffi_cast('char *', new_frame.buf)
                            for i = 0, new_frame.count - 1 do
                                ffi.copy(ts_ptr, p, ffi.sizeof('int64_t'))
                                local ts = tonumber(ts_ptr[0]) / 40000000.0
                                local s, f = math.modf(ts)
                                s_ptr[0], f_ptr[0] = s, math.floor(f * (2^32))
                                ffi.copy(p, s_ptr, ffi.sizeof('int32_t'))
                                ffi.copy(p + ffi.sizeof('int32_t'), f_ptr,
                                    ffi.sizeof('uint32_t'))
                                p = p + frame.element_size
                            end

                            -- Send the frame on the stream
                            ts = new_frame.hdr.ts
                            local pkt = ffi_cast('hbm_stream_packet_t *',
                                new_frame.hdr)
                            pkt.hdr = htonl(C.HDR_DATA + sid)
                            pkt.length =
                                htonl(new_frame.count * new_frame.element_size)
                            cinfo.ws:send(pkt,(new_frame.count *
                                new_frame.element_size) + spl)
                            Frame_delete(new_frame)
                            new_frame = nil
                        else
                            -- Sync channel frames just get sent
                            ts = frame.hdr.ts
                            local pkt = ffi_cast('hbm_stream_packet_t *',
                                frame.hdr)
                            pkt.hdr = htonl(C.HDR_DATA + sid)
                            pkt.length = htonl(frame.count * frame.element_size)
                            cinfo.ws:send(pkt,
                                (frame.count * frame.element_size) + spl)
                        end
                        frame.hdr.ts = ts
                    end
                    frame = frame.prev
                end

                if have_min_max then
                    self:send_metadata(cinfo, sid, cjson.encode({
                            method = 'min_max',
                            params = {
                                min = tracked_min,
                                max = tracked_max
                            },
                        }))
                end

                -- If a frame was created for the low speed channel delete it.
                if new_frame then Frame_delete(new_frame) end
            end
        end
    end

    -- Flush all of the pipes that were processed
    for _, p in pairs(processed_pipes) do
        Pipe_flush_frames(p)
    end
end

-- Function used to initialize streaming service
function ss:initialize(loop, opts)
    -- Seed the random number generator
    math.randomseed(os.time())

    opts = opts or {}
    opts.rate = opts.rate or .05

    -- Save off stuff
    self.loop = loop
    self.client_list = {}

    self.channel_list = {}
    self.channel_info = {}
    self.max_sid = 0

    -- Create an event based HTTP server for stream control
    self.http_server = http_server:new{
        ev_loop = loop,
        handler = self.http_handler,
        add_to_watchers = {
            me = self,
        }
    }

    -- FIXME?:  Should this be limited to the "external" interface only?
    self.http_w = self.http_server:listen({address = '*', port = opts.cport })
    self.http_port = self.http_w.port

    -- Save HTTP info needed when websocket clients connect
    self.http_server_info = {
        httpPath = '/stream_control',
        httpMethod = 'POST',
        httpVersion = '1.1',
        port = self.http_w.port,
    }

    self.ws_port = opts.port

    -- create a event based websocket server and start listening
    self.ws_server = tcp_server:new(ev.Loop.default, self.ws_port)
    self.ws_server:on_new_client(function(...) self:ws_connect(...) end)
    self.ws_server:start()

    -- create a event based TCP server and start listening
    self.tcp_server = tcp_server:new(ev.Loop.default, 1081)
    self.tcp_server:on_new_client(function(...) self:tcp_connect(...) end)
    self.tcp_server:start()

    -- Create a timer used to move sample data from the input pipes to the
    -- stream sockets
    self.data_writer = ev.Timer.new(function(l, t, e)
            self:process_streams()
        end, opts.rate, opts.rate):start(loop)
end

-- Terminate the websocket stuff
function ss:terminate(log, close_complete_func)
    -- For every client connected, close the connection and leave.  NOTICE
    -- this is intend for use just before using process termination
--[[
    if log then
        log:log(log.NOTICE, "Terminating HBM stream server")
    end
]]
    self.client_list = self.client_list or {}
    for i, ci in ipairs(self.client_list) do
        ci.ws:close()
    end
    if close_complete_func then close_complete_func() end
end

return ss
