
-- network_db_access.lua:  Provides functions test engine modules
-- may use to access network database values.
--
-- NOTE:  This currently supports no more than one record per
--        "owner" !

require 'strict'

local m = {}

local __ = require '__'
local ev = require 'ev'
local dbus = require 'dbus'
local ffi = require 'ffi'
local log = require 'log'
local net = require 'net'

local C = ffi.C

local m = {}
local initialized = false


-- Initialize this "singleton" object.
function m:initialize()
    if initialized then
        -- only initialize once
        return false
    end

    self.bus = dbus.bus_get('system')
    assert (self.bus, "Could not find 'system' bus with dbus")
    self.bus:hook_libev_default_loop()

    if ev.Loop.default:depth() > 0 then
        self.network =
            assert(self.bus:auto_proxy('com.somat.armada.Network',
                '/com/somat/armada/Network'))
    else
        self.network =
            assert(self.bus:auto_proxy_wait('com.somat.armada.Network',
                '/com/somat/armada/Network'))
    end
    assert (self.network, "Could not find Network")

    self.rr_changes_bound = false
    self.monitored_rr_owners = {}  -- identifiers of rrs of interest
    self.monitored_rr = {}  -- current state of rrs of interest

    initialized = true
    return true
end

-- Modify record(s).
-- Parameters:
--     'del' are the record(s) to delete
--     'ins' are the record(s) to insert
-- Returns:  success status
function m:modify_rrs(del, ins)
    return self.network:ModifyRRs(del, ins)
end

-- For records with TXT values, modifies the specified record's value
-- by deleting the current record and adding a new one with the
-- desired value.
-- Parameters:
--     'owner' is the name of the record to be changed
--     'val' is the new value
function m:change_rr(owner, val)
    self.network:RemoveLocalRROwner(owner)
    self.network:ModifyRRs({}, {{owner, 'TXT', val}})
end

-- Register with the Network database server to receive
--     notices of changes to the database records.  The
--     flag 'self.rr_changes_bound' prevents registering
--     multiple times.
function m:bind_changed_rrs()
    if not self.rr_changes_bound then
        self.network.ChangedRRs:bind(
            function(removed, added)
                self:changed_rr(removed, added)
            end
        )
        self.rr_changes_bound = true
    end
end

-- Called by the Network ChangedRRs signal to handle changes
--     to database records.
-- Parameters:
--     'removed' are the record(s) deleted
--     'added' are the record(s) to inserted
-- NOTE:  BEWARE of multiple rrs with one owner!
--        This code does not currently handle that case,
--        other than to save (only) the rr with the highest
--        index.
function m:changed_rr(removed, added)
    for i,rr in ipairs(removed) do
        if self.monitored_rr_owners[rr[1]] then
            self.monitored_rr[rr[1]] = nil
        end
    end
    for i,rr in ipairs(added) do
        if self.monitored_rr_owners[rr[1]] then
            self.monitored_rr[rr[1]] = rr
            log:notice(__'"{0}" set to "{1}"',
                rr[1], rr[3])
        end
    end
end

-- Queries the Network database for the current value of
--     specifed records.
-- Parameters:
--     'owner' is the name of the records to be checked
-- NOTE:  BEWARE of multiple rrs with one owner!
--        This code does not currently handle that case,
--        other than to update (only) the rr with the highest
--        index.
function m:update_monitored_rr_values(owner)
    local recs = self.network:QueryRR(owner, 'TXT', '*', '*')
    for i = 1, #recs do
        self.monitored_rr[owner] = recs[i]
    end
end

-- Sets specifed records as monitored for changes.
-- Parameters:
--     'owner' is the name of the records to be monitored
function m:monitor_rr(owner)
    self.monitored_rr_owners[owner] = 1  -- any "true" value will do
end

-- Returns the currently cached value for the specifed records.
-- Parameters:
--     'owner' is the name of the records whose value is to be returned
function m:get_rr(owner)
    return self.monitored_rr[owner]
end

return m
